
# line 2 "hoc.y"
#include "hoc.h"

#define __my_yylex			/* for SGI yacc and C++ */

#if defined(YYDEBUG)
extern int yydebug;
#else
#define YYDEBUG 0	/* so this works with byacc, bison, and yacc */
#endif

/* We need larger internal yacc tables to get consistent results across
   platforms for test/long-operand.hoc */
#define YYMAXDEPTH	8000
#define YYSTACKSIZE	8000

#if defined(HAVE_CTYPE_H)
#include <ctype.h>
#endif

#if defined(HAVE_ERRNO_H)
#include <errno.h>
#endif

#if defined(HAVE_LOCALE_H)
#include <locale.h>
#endif

#if defined(HAVE_SETJMP_H)
#include <setjmp.h>
#endif

#if defined(HAVE_SIGNAL_H)
#include <signal.h>
#endif

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" int yylex ARGS((void));
extern "C" void fpecatch ARGS((int dummy));
extern "C" void intcatch ARGS((int dummy));
extern "C" void xcpucatch ARGS((int dummy));
#else
int yylex ARGS((void));
void fpecatch ARGS((int dummy));
void intcatch ARGS((int dummy));
void xcpucatch ARGS((int dummy));
#endif

#if defined(_AIX)
#undef YYCONST
#define YYCONST
EXTERN void yyerror ARGS((YYCONST char *)); /* otherwise used before prototyped on this system, sigh...*/
#endif

#if defined(HAVE_STDIO_H)
#include <stdio.h>
#endif

#if defined(HAVE_STDLIB_H)
#include <stdlib.h>
#endif

#if defined(HAVE_STRING_H)
#include <string.h>
#endif

#if defined(HAVE_UNISTD_H)
#include <unistd.h>
#endif

#if !defined(R_OK)
#define R_OK 04
#endif

#if !defined(HOCRC)
#define HOCRC	".hocrc"
#endif

#if !defined(SYSHOCDIR)
#define SYSHOCDIR	"/usr/local/share/lib/hoc/hoc-" PACKAGE_VERSION "/"
#endif

#if !defined(SYSHOCHLPBASE)
#define SYSHOCHLPBASE	"help.hoc"
#endif

#if !defined(SYSHOCHLP)
#define SYSHOCHLP SYSHOCDIR "/" SYSHOCHLPBASE
#endif

#if !defined(SYSHOCRCBASE)
#define SYSHOCRCBASE	"hoc.rc"
#endif

#if !defined(SYSHOCRC)
#define SYSHOCRC SYSHOCDIR "/" SYSHOCRCBASE
#endif

#if !defined(SYSHOCXLTBASE)
#define SYSHOCXLTBASE	"translations.hoc"
#endif

#if !defined(SYSHOCXLT)
#define SYSHOCXLT SYSHOCDIR "/" SYSHOCXLTBASE
#endif

#if defined(DEBUG_CODE)
const char *code_argname;
int	debug_code = 0;			/* turn on manually in a debugger seesion */
#define code(c)		(code_argname = #c, (code)(c))
#endif

#define	code2(c1,c2)		code(c1); (void)code(c2)
#define	code3(c1,c2,c3)		code2(c1,c2); (void)code(c3)
#define	code4(c1,c2,c3,c4)	code3(c1,c2,c3); (void)code(c4)
#define CODE(c1)		(void)code(c1)
#define CODE2(c1,c2)		(void)code2(c1,c2)
#define CODE3(c1,c2,c3)		(void)code3(c1,c2,c3)
#define CODE4(c1,c2,c3,c4)	(void)code4(c1,c2,c3,c4)
#define QUOTATION_MARK	'"'

extern const char *	elementsep;	/* defined in io.c */

int			indef;			/* needed in code.c too */
int			inloop;			/* needed in code.c too */
const char *		syshocdir = SYSHOCDIR;

static int		this_c = '\n';		/* file global: used in warning(), yygetid(), yylex() */
static int		infile_count = 0;
static const char *	hocrc = HOCRC;
static const char *	syshochlp = SYSHOCHLP;
static const char *	syshochlpbase = SYSHOCHLPBASE;
static const char *	syshocrc = SYSHOCRC;
static const char *	syshocrcbase = SYSHOCRCBASE;
static const char *	syshocxlt = SYSHOCXLT;
static const char *	syshocxltbase = SYSHOCXLTBASE;

extern FILE *		Fopen ARGS((const char *, const char *, const char **));
extern FILE *		Freopen ARGS((const char *, const char *, FILE*, const char **));

int			main ARGS((int, char *[]));
const char *		yygetstr ARGS((void));		/* used in code.c:varread() */
void			set_filename ARGS((const char *)); /* used in string.c:Load() */

static void             author_and_die ARGS((void));
static int		backslash ARGS((int));
static void             bump_lineno ARGS((void));
static void             copyright_and_die ARGS((void));
static void             defnonly ARGS((const char *));
static void		do_define ARGS((const char *));
static void		do_one_init_file ARGS((const char *filename));
static void		do_undefine ARGS((const char *));
static void             do_init_files ARGS((void));
static void             do_post_args ARGS((int *, char *[]));
static void             do_pre_args ARGS((const int *, const char *[]));
static void		error ARGS((const char *, const char *));
static int              follow ARGS((int, int, int));
static const char *     get_locale_filename ARGS((const char *, const char *));
static const char *     get_locale_name ARGS((void));
static void             help_and_die ARGS((void));
static void             init_lineno ARGS((void));
static void		init_locale ARGS((void));
static void		init_readline ARGS((void));
static void             looponly ARGS((const char *));
static void             show ARGS((const char *[]));
static void             usage ARGS((void));
static void             usage_and_die ARGS((const char *));
static void             version_and_die ARGS((void));


# line 171 "hoc.y"
typedef union
#ifdef __cplusplus
	YYSTYPE
#endif
 {
	Symbol	*sym;	/* symbol table pointer */
	Inst	*inst;	/* machine instruction */
	long	narg;	/* number of arguments */
} YYSTYPE;
# define NUMBER 257
# define STRING 258
# define PRINT 259
# define VAR 260
# define UNDEF 261
# define WHILE 262
# define FOR 263
# define IF 264
# define ELSE 265
# define FUNCTION 266
# define PROCEDURE 267
# define RETURN 268
# define FUNC 269
# define PROC 270
# define READ 271
# define ABORT 272
# define BLTIN0 273
# define BLTIN1 274
# define BLTIN1S 275
# define BLTIN2 276
# define BREAK 277
# define CONTINUE 278
# define DEFINED 279
# define DELETE 280
# define HEX 281
# define HEXFP 282
# define HEXINT 283
# define INDEX 284
# define LENGTH 285
# define NUMTOSTR 286
# define PRINTF 287
# define PRINTLN 288
# define SPRINTF 289
# define STRBLTIN0 290
# define STRBLTIN1 291
# define STRBLTIN1N 292
# define STRBLTIN2 293
# define STRVAR 294
# define STRFTIME 295
# define STRTONUM 296
# define STRTOSYMNUM 297
# define STRTOSYMSTR 298
# define SUBSTR 299
# define ARG 300
# define STRARG 301
# define ADDEQ 302
# define SUBEQ 303
# define MULEQ 304
# define DIVEQ 305
# define MODEQ 306
# define CONSTEQ 307
# define OR 308
# define AND 309
# define GT 310
# define GE 311
# define LT 312
# define LE 313
# define EQ 314
# define NE 315
# define LTGT 316
# define UNARYMINUS 317
# define UNARYPLUS 318
# define NOT 319
# define INC 320
# define DEC 321
# define TO_POWER 322

#include <inttypes.h>

#ifdef __STDC__
#include <stdlib.h>
#include <string.h>
#define	YYCONST	const
#else
#include <malloc.h>
#include <memory.h>
#define	YYCONST
#endif

#include <values.h>

#if defined(__cplusplus) || defined(__STDC__)

#if defined(__cplusplus) && defined(__EXTERN_C__)
extern "C" {
#endif
#ifndef yyerror
#if defined(__cplusplus)
	void yyerror(YYCONST char *);
#endif
#endif
#ifndef yylex
	int yylex(void);
#endif
	int yyparse(void);
#if defined(__cplusplus) && defined(__EXTERN_C__)
}
#endif

#endif

#define yyclearin yychar = -1
#define yyerrok yyerrflag = 0
extern int yychar;
extern int yyerrflag;
YYSTYPE yylval;
YYSTYPE yyval;
typedef int yytabelem;
#ifndef YYMAXDEPTH
#define YYMAXDEPTH 150
#endif
#if YYMAXDEPTH > 0
int yy_yys[YYMAXDEPTH], *yys = yy_yys;
YYSTYPE yy_yyv[YYMAXDEPTH], *yyv = yy_yyv;
#else	/* user does initial allocation */
int *yys;
YYSTYPE *yyv;
#endif
static int yymaxdepth = YYMAXDEPTH;
# define YYERRCODE 256

# line 446 "hoc.y"

	/* end of grammar */

#ifndef YYCONST
#define YYCONST				/* symbol used by Sun Solaris yacc */
#endif

#if defined(__osf__)
EXTERN void yyerror ARGS((YYCONST char *));	/* keep C++ compilers happy */
#endif

#include "readline.h"

#undef getc
#define getc(fpin)			irl_getchar(fpin)
#undef ungetc
#define ungetc(c,fpin)			irl_unget_char(c)
#undef fscanf
#define fscanf(fpin,format,pdata)	irl_fscanf(fpin,format,pdata)

#define ARGMATCH(arg,option,minmatch) (strncmp(arg,option,MAX(strlen(arg),minmatch)) == 0)
#undef MAX				/* because some systems define this already */
#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define OPTCHAR		'-'

extern FILE *fplog;

int cd_pwd_enabled = 1;	  		/* reset by --no-cd and --secure command-line options */
int file_trace_enabled = 1;		/* reset by --secure command-line option */
int load_enabled = 1;			/* reset by --no-load and --secure command-line options */
int logfile_enabled = 1;		/* reset by --no-logfile and --secure command-line options */
int save_enabled = 1;			/* reset by --no-save and --secure command-line options */
int use_readline = 1;			/* reset by --no-readline command-line option */
static int print_banner = 1;   		/* reset by --no-banner command-line option */
static int read_help_file = 1;		/* reset by --no-help-file command-line option */
static int read_site_file = 1;		/* reset by --no-site-file command-line option */
static int read_translation_file = 1;	/* reset by --no-translation-file command-line option */
static int read_user_file = 1;		/* reset by --no-user-file command-line option */
static int verbose_mode = 1;   		/* reset by --silent command-line option */

static jmp_buf		begin;
static int		gargc;
static char		**gargv;	/* global argument list */
static const char	*infile;	/* input file name */
static int		lineno = 1;
const char		*progname;

FILE			*fin;		/* input file pointer (also used in code.c and string.c) */

static void
author_and_die(void)
{
	static const char *text[] =
	{
		PACKAGE_NAME, " was written by\n\n",

		"\tBrian W. Kernighan and Rob Pike\n\n",

		"and described in their book:\n\n",

		"\tThe UNIX Programming Environment\n",
		"\tPrentice-Hall\n",
		"\t1984\n",
		"\tpp. xii + 357\n",
		"\tISBN 0-13-937699-2 (hardcover), 0-13-937681-X (paperback)\n",
		"\tLCCN QA76.76.O63 K48 1984\n\n",

		"Extensions for IEEE 754 arithmetic, help, additional constants,\n",
		"statements, and functions, initialization files, internationalization,\n",
		"and GNU readline support were added by:\n\n",

		"\t", PACKAGE_BUGREPORT, "\n\n",

		"The extended hoc master Internet source distribution site is at:\n\n",

		"\tftp://ftp.math.utah.edu/pub/hoc\n",
		"\thttp://www.math.utah.edu/pub/hoc/\n\n",

		(const char *)NULL,
	};
	show(text);
	exit(EXIT_SUCCESS);
}

static int
backslash(int c)	/* get next char with \'s interpreted */
{
	static char transtab[] = "a\007b\bf\fn\nr\rt\tv\v"; /* K&R and Standard C escapes */

#define is_odigit(x) (isdigit(x) && (x != '8') && (x != '9'))

	if (c != '\\')
		return c;
	c = getc(fin);
	if (c == '\n')
		bump_lineno();
	if (islower(c) && (strchr(transtab, c) != (char *)NULL))
		return strchr(transtab, c)[1];
	else if (c == 'E')		/* \E -> ESCape */
	{
		/* NB: Standard C reserves uppercase escapes to
		   implementation-specific extensions */

		return ('\033');	/* ASCII ESCape */
	}
	else if (is_odigit(c))
	{		 /* \o, \oo, \ooo -> octal character number */
		int n;
		int value;

		n = 0;
		value = 0;
		while (is_odigit(c) && ((++n) <= 3))
		{
			value = 8*value + (c - (int)'0');
			c = getc(fin);
		}
		(void)ungetc(c,fin);
		return (value);
	}
	else if (c == (int)'x')
	{	/* \xhhhh... (any number of h's) -> hexadecimal character number */
		int value;

		value = 0;
		c = getc(fin);
		while (isxdigit(c))
		{
			if (isupper(c))
				c = tolower(c);
			if (((int)'a' <= c) && (c <= (int)'f'))
				value = 16*value + (c - (int)'a' + 10);
			else if (isdigit(c))
				value = 16*value + (c - (int)'0');
			c = getc(fin);
		}
		(void)ungetc(c,fin);
		return (value);
	}
	return c;

#undef is_odigit
}

static void
bump_lineno(void)
{
#if 0
	/* temporarily suppressed: see io.c:run3() header comments for why */

	static fp_t * pdlineno = (fp_t*)NULL;

	if (pdlineno == (fp_t*)NULL)
	{
		Symbol *s;

		s = lookup("__LINENO__");
		if (s == (Symbol*)NULL)
			s = install_number("__LINENO__", FP(0.0));
		make_immutable(s); /* to prevent user code assignments */
		pdlineno = &s->u.val;
	}
	lineno++;
	*pdlineno = (fp_t)lineno;	/* update __LINENO__ value in symbol table */
#else
	/* We need this alternate version of line number incrementing to match code in io.c:run3() */
	Symbol *s;
	s = lookup("__LINENO__");
	if (s == (Symbol*)NULL)
		s = install_number("__LINENO__", FP(0.0));
	if (lineno == -1)		/* special case of new input file */
		s->u.val = (fp_t)lineno;
	s->u.val += FP(1.0);
	lineno = (int)(s->u.val);
#endif
}

static void
copyright_and_die(void)
{
	static const char *text[] =
	{
		"\n",
		"=================================================================\n",
		"Copyright (C) AT&T 1995\n",
		"All Rights Reserved\n\n",

		"Permission to use, copy, modify, and distribute this software and\n",
		"its documentation for any purpose and without fee is hereby\n",
		"granted, provided that the above copyright notice appear in all\n",
		"copies and that both that the copyright notice and this\n",
		"permission notice and warranty disclaimer appear in supporting\n",
		"documentation, and that the name of AT&T or any of its entities\n",
		"not be used in advertising or publicity pertaining to\n",
		"distribution of the software without specific, written prior\n",
		"permission.\n\n",

		"AT&T DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,\n",
		"INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.\n",
		"IN NO EVENT SHALL AT&T OR ANY OF ITS ENTITIES BE LIABLE FOR ANY\n",
		"SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES\n",
		"WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER\n",
		"IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,\n",
		"ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF\n",
		"THIS SOFTWARE.\n",
		"=================================================================\n\n",

		(const char *)NULL,
	};
	show(text);
	exit(EXIT_SUCCESS);
}

static void
defnonly(const char *s)	/* warn if illegal definition */
{
	if (!indef)
		execerror(s, "used outside definition");
}

static void
do_define(const char *s)
{	/* expect name (== name=1) or name=numericvalue or name=name or name="string" */
	/* (= may be replaced by := for permanent assignments) */
	char *equals;
	char *t;

	t = (char *)dupstr(s);
	equals = strchr(t,'=');
	if (equals == (char *)NULL)
	{
		if (is_immutable(t))
			error("illegal reassignment to immutable named constant", s);
		else
			(void)update_number(t,FP(1.0));	/* -Dname means -Dname=1 */
	}
	else					/* have -Dname=something or -Dname:=permanent-something */
	{
		char *value;
		int immutable;

		immutable = (equals[-1] == ':');

		if (immutable)
			equals[-1] = '\0';
		else
			equals[0] = '\0';

		if (is_immutable(t))
			error("illegal reassignment to immutable named constant", s);

		value = equals + 1;
		if (*value == QUOTATION_MARK) /* have name="value" */
		{
			char *q;

			value++;
			q = strchr(value, QUOTATION_MARK);
			if (q == (const char *)NULL)
				error("unclosed quotation mark in command-line option", s);
			else if (q[1] == '\0')
			{
				const char *v;

				*q = '\0';
				v = Expand(value);
				if (immutable)
					(void)update_const_string(t,v);
				else
					(void)update_string(t,v);
				efree((void *)v);
			}
			else
				error("unrecognized string in definition", s);
		}
		else if (IsIdStart(*value)) /* have name=name */
		{
			Symbol *sp;

			sp = lookup(value);
			if (sp == (Symbol*)NULL)
				error("unknown symbol in definition", s);
			else if (sp->type == VAR)
			{
				if (immutable)
					(void)update_const_number(t, sp->u.val);
				else
					(void)update_number(t, sp->u.val);
			}
			else if (sp->type == STRVAR)
			{
				if (immutable)
					(void)update_const_string(t, sp->u.str);
				else
					(void)update_string(t, sp->u.str);
			}
			else
				error("illegal symbol in definition", s);
		}
		else /* expect name=numeric-value */
		{
			char *endptr;
			fp_t d;

			d = strton(value, &endptr);
			if (*endptr == '\0')
			{
				if (immutable)
					(void)update_const_number(t, d);
				else
					(void)update_number(t, d);
			}
			else
				error("unrecognized number in definition", s);
		}
	}
	efree((void *)t);
}

static void
do_init_files(void)
{
	/* Process the optional startup files in documented
	   (in "man hoc") order of use:
		(1) system hoc.rc
		(2) locale hoc.rc
		(3) system help.hoc
		(4) locale help.hoc
		(5) system translations.hoc
		(6) locale translations.hoc
		(7) user .hocrc
	*/

	const char *p;

	if (read_site_file)
	{
		do_one_init_file(syshocrc);
		p = get_locale_filename(syshocdir, syshocrcbase);
		do_one_init_file(p);
		efree((void*)p);
	}

	if (read_help_file)
	{
		do_one_init_file(syshochlp);
		p = get_locale_filename(syshocdir, syshochlpbase);
		do_one_init_file(p);
		efree((void*)p);
	}

	if (read_translation_file)
	{
		do_one_init_file(syshocxlt);
		p = get_locale_filename(syshocdir, syshocxltbase);
		do_one_init_file(p);
		efree((void*)p);
	}

#if defined(HAVE_UNIX_HOME_DIRECTORY)
	if (read_user_file)
	{
		const char * userhocrc;
		const char * home;

		home = getenv("HOME");
		if (home == (const char *)NULL)
			return;
		userhocrc = concat3(home, "/", hocrc);
		do_one_init_file(userhocrc);
		efree((void *)userhocrc);
	}
#endif
}

static void
do_one_init_file(const char *filename)
{
	const char *new_filename;

	/* If filename is readable, attempt to open it and process it.  Otherwise,
	   return silently. */
	if ((filename != (const char *)NULL) && (access(filename, R_OK) == 0))
	{
		fin = Fopen(filename, "r", &new_filename);
		if (fin == (FILE*)NULL)
		{
			(void)fprintf(stderr,
				      msg_translate("%s: can't open %s\n"),
				      progname, filename);
			efree((void *)new_filename);
		}
		else
		{
			set_filename(new_filename);
			efree((void *)new_filename);
			run();
			(void)fclose(fin);
			fin = (FILE*)NULL;
		}
	}
}

static void
do_post_args(int *pargc, char * argv[])
{
	int k, m;
	char *p;

	/* Process command-line arguments after initialization files
	   have been read.  Any options that are recognized are
	   removed from argv[], and *pargc is reduced accordingly. */

	for (k = 1; k < *pargc; ++k)
	{
		p = argv[k];
		if (p[0] == OPTCHAR)
		{
			p++;
			if (p[0] == OPTCHAR)
				++p;	/* support GNU/POSIX style --option */

			if (STREQUAL(p,"")) /* ignore "-" option (meaning, stdin) */
			{
				if (p == (argv[k] + 2))
					p[-1] = '\0'; /* convert "--" to "-" */
				continue;
			}
			else if (*p == 'D')
				noop();	/* already handled in do_pre_args() */
			else if (*p == 'U')
				noop();	/* already handled in do_pre_args() */
			else if (ARGMATCH(p,"author",1))
				author_and_die();
			else if (ARGMATCH(p,"copyright",1))
				copyright_and_die();
			else if (ARGMATCH(p,"?",1))
				help_and_die();
			else if (ARGMATCH(p,"help",1))
				help_and_die();
			else if (ARGMATCH(p,"no-banner",4))
			{
				print_banner = 0;
				(void)update_number("__BANNER__", (fp_t)print_banner);
			}
			else if (ARGMATCH(p,"no-cd",4))
				cd_pwd_enabled = 0;
			else if (ARGMATCH(p,"no-help-file",4))
				read_help_file = 0;
			else if (ARGMATCH(p,"no-load",6))
				load_enabled = 0;
			else if (ARGMATCH(p,"no-logfile",6))
				logfile_enabled = 0;
			else if (ARGMATCH(p,"no-readline",4))
			{
				use_readline = 0;
				(void)update_number("__READLINE__", (fp_t)use_readline);
			}
			else if (ARGMATCH(p,"no-save",5))
				save_enabled = 0;
			else if (ARGMATCH(p,"no-site-file",5))
				read_site_file = 0;
			else if (ARGMATCH(p,"no-translation-file",4))
				read_translation_file = 0;
			else if (ARGMATCH(p,"no-user-file",4))
				read_user_file = 0;
			else if (ARGMATCH(p,"quick",1))
			{
				read_help_file = 0;
				read_site_file = 0;
				read_translation_file = 0;
				read_user_file = 0;
			}
			else if (ARGMATCH(p,"secure",2))
			{
				cd_pwd_enabled = 0;
				load_enabled = 0;
				logfile_enabled = 0;
				save_enabled = 0;
				file_trace_enabled = 0;
			}
			else if (ARGMATCH(p,"silent",1)) /* NB: can be abbreviated to 1 char */
			{
				verbose_mode = 0;
				(void)update_number("__VERBOSE__", (fp_t)verbose_mode);
			}
			else if (ARGMATCH(p,"trace-file-opening",1))
				(void)update_number("__DEBUG_OPEN__", FP(1.0));
			else if (ARGMATCH(p,"version",1))
				version_and_die();
			else
				usage_and_die(argv[k]);
			/* Remove the option from the argument list */
			for (m = k + 1; m < (*pargc); ++m)
				argv[m-1] = argv[m];
			(*pargc)--;
			k--;	/* avoid missing the next argument */
		}
	}
	argv[*pargc] = (char *)NULL;	/* ANSI C89 Section 2.1.2.2.1: ``argv[argc] shall be a NULL pointer'' */

#if !defined(HAVE_GNU_READLINE)
	use_readline = 0;
	(void)update_number("__READLINE__", (fp_t)use_readline);
#endif

	make_immutable(lookup("__READLINE__"));	/* this decision may someday be changed */
}

static void
do_pre_args(const int *pargc, const char * argv[])
{
	int k;
	const char *p;

	(void)update_string("__FILE__", "command-line"); /* in case error() is called */

	/* Process those command-line arguments that need to be seen
	   before initialization files are read.  No errors are raised
	   here for unrecognized arguments; that will be done in
	   do_post_args().  *pargc and argv[] are left intact. */

	for (k = 1; k < *pargc; ++k)
	{
		p = argv[k];
		if (p[0] == OPTCHAR)
		{
			p++;
			if (p[0] == OPTCHAR)
				++p; /* support GNU/POSIX style --option */

			if (ARGMATCH(p,"no-banner",4))
			{
				print_banner = 0;
				(void)update_number("__BANNER__", (fp_t)print_banner);
			}
			else if (ARGMATCH(p,"no-help-file",4))
				read_help_file = 0;
			else if (ARGMATCH(p,"no-logfile",6))
				logfile_enabled = 0;
			else if (ARGMATCH(p,"no-readline",4))
			{
				use_readline = 0;
				(void)update_number("__READLINE__", (fp_t)use_readline);
			}
			else if (ARGMATCH(p,"no-site-file",5))
				read_site_file = 0;
			else if (ARGMATCH(p,"no-translation-file",4))
				read_translation_file = 0;
			else if (ARGMATCH(p,"no-user-file",4))
				read_user_file = 0;
			else if (ARGMATCH(p,"quick",1))
			{
				read_help_file = 0;
				read_site_file = 0;
				read_translation_file = 0;
				read_user_file = 0;
			}
			else if (ARGMATCH(p,"silent",1)) /* can be abbreviated to 1 character */
			{
				verbose_mode = 0;
				(void)update_number("__VERBOSE__", (fp_t)verbose_mode);
			}
			else if (ARGMATCH(p,"trace-file-opening",1))
				(void)update_number("__DEBUG_OPEN__", FP(1.0));
			else if (*p == 'D')
				do_define(p+1);
			else if (*p == 'U')
				do_undefine(p+1);
		}
	}
}

static void
do_undefine(const char *s)
{
	if (!delete_symbol(lookup(s)))
		error("deletion failed for symbol", s);
}

static void
error(const char *s, const char *t)
{
	warning(s,t);
	exit(EXIT_FAILURE);
}

void
execerror(const char * s, const char * t)	/* recover from run-time error */
{
	warning(s, t);
	(void)fseek(fin, 0L, 2);	/* flush rest of file */
	longjmp(begin, 0);
}

#if defined(__sgi)
#include <sys/fpu.h>

static void
flush_to_zero(int on_off)		/* see "man sigfpe" on SGI IRIX 6.x for documentation */
{
	union fpc_csr n;

	n.fc_word = get_fpc_csr();
	n.fc_struct.flush = (on_off ? 1 : 0);
	set_fpc_csr(n.fc_word);
}
#endif

static int
follow(int expect, int ifyes, int ifno)	/* look ahead for >=, etc. */
{
	int ch = getc(fin);

	if (ch == expect)
		return ifyes;
	(void)ungetc(ch, fin);
	return ifno;
}

void
fpecatch(int dummy)	/* catch floating point exceptions */
{
	execerror("floating point exception", (const char *)NULL);
}

static const char *
get_locale_filename(const char *dirname, const char *basename)
{
	/* Given dirname = "/x/y/z" and basename == "foo.bar", return a dynamic
	   copy of "/x/y/z/locale/LOCALENAME/foo.bar", where LOCALENAME is
	   obtained from get_locale_name() */
	const char *locale;

	locale = get_locale_name();
	if (locale == (const char *)NULL)
		return ((const char *)NULL);
	else
		return (concat5(dirname, "/locale/", locale, "/", basename));
}

static const char *
get_locale_name(void)
{
	/* According to "man 3c setlocale", the language of text
	messages is determined by the locale variable LC_MESSAGES: its
	value is set by the first-defined of three environment
	variables.  Return a static copy of the value. */

	const char *locale_name;

	locale_name = getenv("LC_ALL");
	if (locale_name == (const char *)NULL)
		locale_name = getenv("LC_MESSAGES");
	if (locale_name == (const char *)NULL)
		locale_name = getenv("LANG");
	return (locale_name);
}

const char *
get_prompt(void)
{
	Symbol *s;

	s = lookup("__PROMPT__");
	if (s == (Symbol*)NULL)
		s = install_string("__PROMPT__", (PACKAGE_NAME "> "));
	return (s->u.str);
}

int
get_verbose(void)
{
	Symbol *s;

	s = lookup("__VERBOSE__");
	if (s == (Symbol*)NULL)
		(void)install_number("__VERBOSE__", (fp_t)verbose_mode);
	else
		verbose_mode = (s->u.val != FP(0.0));
	return (verbose_mode);
}

static void
help_and_die(void)
{
	static const char *text[] =
	{
		"\n",
		"For additional help, try ", PACKAGE_NAME, "'s help() command, or in a shell,\n",
		"use the UNIX manual page command, man ", PACKAGE_NAME, ".\n",

		(const char *)NULL,
	};

	usage();
	show(text);
	exit(EXIT_SUCCESS);
}

static void
init_fp_system(void)
{
	/* Do an system-dependent initializations of the
	   floating-point arithmetic system to enable full IEEE 754
	   support, if possible. */

#if defined(__sgi)
	flush_to_zero(0);		/* to get support for subnormals! */
#endif

}

static void
init_limits(void)
{
	/* Set all of the dynamic table sizes so that they can be
	   queried even before their corresponding arrays have been
	   allocated. */
	(void)update_const_number("__MAX_FRAME__", FP(0.0));
	(void)update_const_number("__MAX_LINE__", FP(0.0));
	(void)update_const_number("__MAX_NAME__", FP(0.0));
	(void)update_const_number("__MAX_PROG__", FP(0.0));
	(void)update_const_number("__MAX_PUSHBACK__",FP(0.0));
	(void)update_const_number("__MAX_STACK__", FP(0.0));
	(void)update_const_number("__MAX_STRING__", FP(0.0));
	(void)update_const_number("__MAX_TOKEN__", FP(0.0));
}

static void
init_lineno(void)
{
	EOF_flag = 0;

	/* Because hoc evaluation happens AFTER complete lines have
	   been read, we start lineno at 0, so that, e.g.,

		println __LINENO__

	on line 1 will print 1, not 2. */

	lineno = -1;
	bump_lineno();
}

static void
init_locale(void)
{
#if defined(HAVE_LOCALE_H)
	/* These calls internationalize the hoc language!  However, in
	   order for existing programs to work, numbers MUST use a
	   decimal point, NOT a comma (or other character).  Without
	   this setting, strtod(), fscanf(), and *printf() all alter
	   number formats to locale-specific conventions.  Similarly,
	   settings other than "C" would violate assumptions about
	   sorting in the who() function, and time stamps in the now()
	   function. */

	const char *locale_name;

	locale_name = get_locale_name();
	if (locale_name != (const char *)NULL)
		(void)setlocale(LC_ALL, locale_name);
	(void)setlocale(LC_COLLATE,	"C");
	(void)setlocale(LC_CTYPE,	"C");
	(void)setlocale(LC_MONETARY,	"C");

#if defined(LC_MESSAGES)		/* POSIX.2 extension beyond C89, C++98, and C99 */
	(void)setlocale(LC_MESSAGES,	"C");
#endif

	(void)setlocale(LC_NUMERIC,	"C");
	(void)setlocale(LC_TIME,	"C");
#endif /* defined(HAVE_LOCALE_H) */
}

static void
init_readline(void)
{
#if defined(HAVE_GNU_READLINE)
	(void)rl_bind_key ('\t', rl_insert);	/* TAB is normal, NOT filename completer */
	(void)rl_bind_key ('\033', rl_complete); /* put filename completion on ESCape */
	irl_fd_stdin = dup(fileno(stdin)); /* save handle for original stdin */
#endif
}

static void
init_symbols(void)
{
	(void)update_number("__BANNER__", (fp_t)print_banner); /* so -no-banner setting is available to user code */
	(void)update_number("__READLINE__", (fp_t)use_readline); /* so -no-readline setting is available to user code */
	(void)update_number("__VERBOSE__",(fp_t)verbose_mode); /* so -silent setting is available to user code */

#if defined(HAVE_IEEE_754)
	(void)install_const_number("__IEEE_754__", FP(1.0));
#else
	(void)install_const_number("__IEEE_754__", FP(0.0));
#endif

#if defined(HAVE_ACOSH)
	(void)install_const_number("__NATIVE_ACOSH__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_ACOSH__", FP(0.0));
#endif

#if defined(HAVE_ACOSH)
	(void)install_const_number("__NATIVE_ACOSH__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_ACOSH__", FP(0.0));
#endif

#if defined(HAVE_ASINH)
	(void)install_const_number("__NATIVE_ASINH__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_ASINH__", FP(0.0));
#endif

#if defined(HAVE_ATANH)
	(void)install_const_number("__NATIVE_ATANH__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_ATANH__", FP(0.0));
#endif

#if defined(HAVE_CBRT)
	(void)install_const_number("__NATIVE_CBRT__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_CBRT__", FP(0.0));
#endif

#if defined(HAVE_CEIL)
	(void)install_const_number("__NATIVE_CEIL__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_CEIL__", FP(0.0));
#endif

#if defined(HAVE_COPYSIGN)
	(void)install_const_number("__NATIVE_COPYSIGN__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_COPYSIGN__", FP(0.0));
#endif

#if defined(HAVE_ERF)
	(void)install_const_number("__NATIVE_ERF__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_ERF__", FP(0.0));
#endif

#if defined(HAVE_ERFC)
	(void)install_const_number("__NATIVE_ERFC__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_ERFC__", FP(0.0));
#endif

#if defined(HAVE_EXPM1)
	(void)install_const_number("__NATIVE_EXPM1__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_EXPM1__", FP(0.0));
#endif

#if defined(HAVE_FLOOR)
	(void)install_const_number("__NATIVE_FLOOR__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_FLOOR__", FP(0.0));
#endif

#if defined(HAVE_FMOD)
	(void)install_const_number("__NATIVE_FMOD__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_FMOD__", FP(0.0));
#endif

#if defined(HAVE_GAMMA)
	(void)install_const_number("__NATIVE_GAMMA__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_GAMMA__", FP(0.0));
#endif

#if defined(HAVE_GETGID)
	(void)install_const_number("__NATIVE_GETGID__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_GETGID__", FP(0.0));
#endif

#if defined(HAVE_GETPID)
	(void)install_const_number("__NATIVE_GETPID__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_GETPID__", FP(0.0));
#endif

#if defined(HAVE_GETPPID)
	(void)install_const_number("__NATIVE_GETPPID__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_GETPPID__", FP(0.0));
#endif

#if defined(HAVE_GETUID)
	(void)install_const_number("__NATIVE_GETUID__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_GETUID__", FP(0.0));
#endif

#if defined(HAVE_HYPOT)
	(void)install_const_number("__NATIVE_HYPOT__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_HYPOT__", FP(0.0));
#endif

#if defined(HAVE_ILOGB)
	(void)install_const_number("__NATIVE_ILOGB__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_ILOGB__", FP(0.0));
#endif

#if defined(HAVE_J0)
	(void)install_const_number("__NATIVE_J0__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_J0__", FP(0.0));
#endif

#if defined(HAVE_J1)
	(void)install_const_number("__NATIVE_J1__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_J1__", FP(0.0));
#endif

#if defined(HAVE_JN)
	(void)install_const_number("__NATIVE_JN__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_JN__", FP(0.0));
#endif

#if defined(HAVE_LDEXP)
	(void)install_const_number("__NATIVE_LDEXP__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_LDEXP__", FP(0.0));
#endif

#if defined(HAVE_LGAMMA)
	(void)install_const_number("__NATIVE_LGAMMA__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_LGAMMA__", FP(0.0));
#endif

#if defined(HAVE_LOG1P)
	(void)install_const_number("__NATIVE_LOG1P__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_LOG1P__", FP(0.0));
#endif

#if defined(HAVE_NEAREST)
	(void)install_const_number("__NATIVE_NEAREST__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_NEAREST__", FP(0.0));
#endif

#if defined(HAVE_NEXTAFTER)
	(void)install_const_number("__NATIVE_NEXTAFTER__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_NEXTAFTER__", FP(0.0));
#endif

#if defined(HAVE_REMAINDER)
	(void)install_const_number("__NATIVE_REMAINDER__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_REMAINDER__", FP(0.0));
#endif

#if defined(HAVE_RINT)
	(void)install_const_number("__NATIVE_RINT__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_RINT__", FP(0.0));
#endif

#if defined(HAVE_RSQRT)
	(void)install_const_number("__NATIVE_RSQRT__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_RSQRT__", FP(0.0));
#endif

#if defined(HAVE_SCALB)
	(void)install_const_number("__NATIVE_SCALB__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_SCALB__", FP(0.0));
#endif

#if defined(HAVE_SIGNIFICAND)
	(void)install_const_number("__NATIVE_SIGNIFICAND__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_SIGNIFICAND__", FP(0.0));
#endif

#if defined(HAVE_SNPRINTF)
	(void)install_const_number("__NATIVE_SNPRINTF__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_SNPRINTF__", FP(0.0));
#endif

#if defined(HAVE_STRFTIME)
	(void)install_const_number("__NATIVE_STRFTIME__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_STRFTIME__", FP(0.0));
#endif

#if defined(HAVE_STRTOLD)
	(void)install_const_number("__NATIVE_STRTOLD__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_STRTOLD__", FP(0.0));
#endif

#if defined(HAVE_TRUNC)
	(void)install_const_number("__NATIVE_TRUNC__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_TRUNC__", FP(0.0));
#endif

#if defined(HAVE_Y0)
	(void)install_const_number("__NATIVE_Y0__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_Y0__", FP(0.0));
#endif

#if defined(HAVE_Y1)
	(void)install_const_number("__NATIVE_Y1__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_Y1__", FP(0.0));
#endif

#if defined(HAVE_YN)
	(void)install_const_number("__NATIVE_YN__", FP(1.0));
#else
	(void)install_const_number("__NATIVE_YN__", FP(0.0));
#endif

	(void)install_const_string("__PACKAGE_BUGREPORT__",	PACKAGE_BUGREPORT);
	(void)install_const_string("__PACKAGE_DATE__",		PACKAGE_DATE);
	(void)install_const_string("__PACKAGE_NAME__",		PACKAGE_NAME);
	(void)install_const_string("__PACKAGE_STRING__",	PACKAGE_STRING);
	(void)install_const_string("__PACKAGE_VERSION__",	PACKAGE_VERSION);

	/* Record the initialization file directories and names for
	   use in the language-changing procedures */

	(void)install_const_string("__HOCRC__",		hocrc);
	(void)install_const_string("__SYSHOCDIR__",	syshocdir);
	(void)install_const_string("__SYSHOCHLPBASE__",	syshochlpbase);
	(void)install_const_string("__SYSHOCHLP__",	syshochlp);
	(void)install_const_string("__SYSHOCRCBASE__",	syshocrcbase);
	(void)install_const_string("__SYSHOCRC__",	syshocrc);
	(void)install_const_string("__SYSHOCXLTBASE__",	syshocxltbase);
	(void)install_const_string("__SYSHOCXLT__",	syshocxlt);

	{
		const char *p1;
		const char *p2;
		const char *p3;
		const char *p4;
		(void)install_const_string("__SYSHOCPATH__",
					   p1 = concat5((p2 = concat2(syshocdir, "/site")),
							elementsep,
							(p3 = concat2((p4 = parentdir(syshocdir)), "/site")),
							elementsep,
							syshocdir));
		efree((void *)p1);
		efree((void *)p2);
		efree((void *)p3);
		efree((void *)p4);
	}

	if (getenv(HOCPATH) == (const char *)NULL) /* then supply HOCPATH=".:" */
	{
		const char *p;

		p = concat2(".", elementsep);
		efree((void*)Putenv(HOCPATH, p));
		efree((void*)p);
	}

#if defined(HAVE_GETGID)
	(void)update_const_number("__GID__", (fp_t)getgid());
#else
	(void)update_const_number("__GID__", FP(0.0));
#endif

#if defined(HAVE_GETPID)
	(void)update_const_number("__PID__", (fp_t)getpid());
#else
	(void)update_const_number("__PID__", FP(0.0));
#endif

#if defined(HAVE_GETPPID)
	(void)update_const_number("__PPID__", (fp_t)getppid());
#else
	(void)update_const_number("__PPID__", FP(0.0));
#endif

#if defined(HAVE_GETUID)
	(void)update_const_number("__UID__", (fp_t)getuid());
#else
	(void)update_const_number("__UID__", FP(0.0));
#endif

}

void
intcatch(int dummy)	/* catch interrupts */
{
	execerror("interrupt", (const char *)NULL);
}

int
main(int argc, char * argv[])	/* hoc7 */
{
	static int first = 1;

	init_limits();			/* this MUST be first! */

#if YYDEBUG
	yydebug = 3;
#endif
	progname = argv[0];
#if 0
	prog = (Inst *)egrow((Inst *)prog, &max_prog, sizeof(prog[0]));
#else
#if 0
	max_prog = 32767;
#else
	max_prog = 8 * 1024 * 1024;
#endif
	prog = (Inst *)emalloc(max_prog * sizeof(prog[0]));
	(void)update_const_number("__MAX_PROG__", (fp_t)max_prog);
#endif
	progp = progbase = &prog[0];

	init_fp_system();
	init_readline();
	init_locale();
	init_builtins();
	init_lineno();
	init_symbols();
	sanity_check();
	do_pre_args(&argc,(const char **)argv);
	do_init_files();
	do_post_args(&argc,argv);
	if (argc == 1) {	/* fake an argument list */
		static char stdhyphen[] = "-";
		static char *stdinonly[] = { stdhyphen, (char *)NULL };

		gargv = stdinonly;
		gargc = 1;
	} else if (first) {	/* for interrupts */
		first = 0;
		gargv = argv+1;
		gargc = argc-1;
	}
	while (moreinput())
		run();
	(void)signal(SIGINT, SIG_IGN);
	irl_free();
	free_symbol_table();
	efree((void*)prog);
	return (EXIT_SUCCESS);
}

int
moreinput(void)
{
	const char *new_filename;

	if (gargc-- <= 0)
		return 0;
	if ((fin != (FILE*)NULL) && (fin != stdin))
	{
		(void)fclose(fin);
		fin = (FILE*)NULL;
	}
	infile = *gargv++;
	if (infile == (const char *)NULL) /* skip over holes from command-line options */
		return moreinput();
	else if (STREQUAL(infile, "-")) {
		fin = stdin;
		new_filename = infile = (const char *)NULL;
#if defined(HAVE_GNU_READLINE)
		(void)dup2(irl_fd_stdin,fileno(stdin)); /* restore original stdin */
	} else if ((fin=Freopen(infile, "r", stdin, &new_filename)) == (FILE*)NULL) {
#else
	} else if ((fin=Fopen(infile, "r", &new_filename)) == (FILE*)NULL) {
#endif
		(void)fprintf(stderr,  msg_translate("%s: can't open %s\n"), progname, infile);
		efree((void *)new_filename);
		return moreinput();
	}
	set_filename((infile == (const char *)NULL) ? "/dev/stdin" : new_filename);
	efree((void *)new_filename);
	return 1;
}

void
run(void)	/* execute until EOF */
{
	Symbol *sp;
	(void)setjmp(begin);
	(void)signal(SIGINT, intcatch);
	(void)signal(SIGFPE, fpecatch);

#if defined(SIGXCPU)
	(void)signal(SIGXCPU, xcpucatch);
#endif

	sp = lookup("__CPU_LIMIT__");
	if (sp != (Symbol*)NULL)
		(void)CPULimit(sp->u.val);
	for (initcode(); yyparse(); initcode())
		execute(progbase);
}

void
set_filename(const char *filename)
{
	Symbol *s;
	char temp[sizeof("__FILE__") + sizeof("18446744073709551616" /* 2^64 */) + 2];

	infile_count++;
	SNPRINTF4(temp, sizeof(temp), "__FILE__[%02d]", infile_count);
	(void)install_const_string(temp,filename);

	s = lookup("__FILE__");
	if (s == (Symbol*)NULL)
	{
#if 0
		/* temporarily suppressed: see io.c:run3() header comments for why */
		(void)install_const_string("__FILE__",filename);
#else
		(void)install_string("__FILE__",filename);
#endif
	}
	else
		(void)set_string(s,filename);
	init_lineno();
}

static void
looponly(const char *s)	/* warn if break/continue statement illegally placed */
{
	if (!inloop)
		execerror(s, "illegal outside loop body");
}

static void
show(const char *text[])
{
	size_t k;

	for (k = 0; text[k] != (const char *)NULL; ++k)
		(void)fprintf(stderr,"%s", msg_translate(text[k]));
}

static void
usage(void)
{
	static const char *text[] =
	{
		"Usage:\n",
		"\t",
		"",
		"\t[ --author ] [ --? ] [ --copyright] [ --Dname ]\n",
		"\t\t[ --Dname=number ] [ --Dname=\"string\" ]\n",
		"\t\t[ --Dname=symbol ] [ --help ] [ --no-banner ]\n",
		"\t\t[ --no-cd ] [ --no-help-file ] [ --no-load ]\n",
		"\t\t[ --no-logfile ] [ --no-readline ] [ --no-save ]\n",
		"\t\t[ --no-site-file ] [ --no-translation-file ]\n",
		"\t\t[ --no-user-file ] [ --quick ] [ --secure ]\n",
		"\t\t[ --silent ] [ --trace-file-opening ][ --Uname ]\n",
		"\t\t[ --version ]\n",
		"\t\tinput-file(s) [ -- ] more-input-file(s)\n\n",
		"Options may be prefixed with either one or two hyphens.\n",
		(const char *)NULL,
	};
	text[2] = progname;		/* cannot be done at compile time */
	show(text);
}

static void
usage_and_die(const char *bad_option)
{
	(void)fprintf(stderr, msg_translate("Unrecognized option: [%s]\n"), bad_option);
	usage();
	exit(EXIT_FAILURE);
}

static void
version_and_die(void)
{
	static const char *text[] =
	{
		"This is ", PACKAGE_NAME,
		" version ", PACKAGE_VERSION,
		" of ", PACKAGE_DATE, ".\n\n",
		"Report bugs to ", PACKAGE_BUGREPORT, "\n",
		(const char *)NULL,
	};
	show(text);
	exit(EXIT_SUCCESS);
}

void
warning(const char *s, const char *t)	/* print warning message */
{
	const char *current_filename;
	(void)fflush(stdout);		/* force out any buffered data */
	(void)fflush(stderr);

	s = msg_translate(s);		/* internationalize the output */
	t = msg_translate(t);

#if 1
	/* Use GNU and UNIX standard
		file:lineno:message
	format instead of old style
		program:message [in file] near line nnn */

	current_filename = get_string("__FILE__");

	(void)fprintf(stderr, "\n");	/* ensure that message starts on a new line! */
	(void)fprintf(stderr, "%s:%d:%s", current_filename, lineno, s);
	if (t != (const char *)NULL)
		(void)fprintf(stderr, " %s", t);
	(void)fprintf(stderr, "\n");

	if (logfile_enabled && (fplog != (FILE*)NULL))
	{
		(void)fprintf(fplog, "#-? %s:%d:%s", current_filename, lineno, s);
		if (t != (const char *)NULL)
			(void)fprintf(fplog, " %s", t);
		(void)fprintf(fplog, "\n");
	}
#else
	(void)fprintf(stderr, "%s: %s", progname, s);
	if (t)
		(void)fprintf(stderr, " %s", t);
	if (infile)
		(void)fprintf(stderr, " in %s", infile);
	(void)fprintf(stderr, " near line %d\n", lineno);
#endif

	while (this_c != '\n' && this_c != EOF)
		if((this_c = getc(fin), this_c) == '\n')	/* flush rest of input line */
			bump_lineno();
		else if (this_c == EOF && errno == EINTR) {
			clearerr(stdin);	/* ick! */
			errno = 0;
		}
	(void)fflush(stderr);
}

void
yyerror(YYCONST char * s)	/* report compile-time error */
{
/*rob
	warning(s, (const char *)NULL);
	longjmp(begin, 0);
rob*/
	execerror(s, (const char *)NULL);
}

Symbol *
yygetid(void)		/* get identifier, installing it in the symbol table if necessary */
{
	Symbol *s;
	static char *sbuf = (char *)NULL;
	static size_t max_sbuf = 0;
	size_t n;

	if (max_sbuf == (size_t)0)	/* guarantee that buffer exists so NUL can be stored */
		sbuf = (char *)emalloc(max_sbuf = (size_t)8);

	this_c = getc(fin);
	n = 0;
	do {
		if ((n + 2) > max_sbuf)
		{
			sbuf = (char *)egrow((void *)sbuf, &max_sbuf, sizeof(sbuf[0]));
			(void)update_const_number("__MAX_NAME__", (fp_t)max_sbuf);
		}
		sbuf[n++] = this_c;
	} while ((this_c=getc(fin)) != EOF && IsIdMiddle(this_c));
	(void)ungetc(this_c, fin);
	sbuf[n] = '\0';
	if ((s=lookup(sbuf),s) == (Symbol*)NULL)
		s = install(sbuf, UNDEF, FP(0.0));

#if defined(DEBUG_YYLEX)
	(void)fprintf(stderr,"yygetid() -> VAR [%s]\n", sbuf);
#endif

	return (s);
}

const char *
yygetstr(void)
{	/* get a quoted string, assuming that the opening quote has just been read */
	static char *sbuf = (char *)NULL;
	static size_t max_sbuf = 0;
	int the_c;
	size_t n;

	if (max_sbuf == (size_t)0)	/* guarantee that buffer exists so NUL can be stored */
		sbuf = (char *)emalloc(max_sbuf = (size_t)16);

	for (n = 0; (the_c = getc(fin)) != QUOTATION_MARK; )
	{
		if (the_c == '\n' || the_c == EOF)
			execerror("missing quote", "");
		if ((n + 2) > max_sbuf)
		{
			sbuf = (char *)egrow((void *)sbuf, &max_sbuf, sizeof(sbuf[0]));
			(void)update_const_number("__MAX_STRING__", (fp_t)max_sbuf);
		}
		sbuf[n++] = backslash(the_c);
	}
	sbuf[n] = '\0';

	return((const char *)&sbuf[0]);
}

int
yylex(void)		/* hoc6 */
{
	while ((this_c=getc(fin)) == ' ' || this_c == '\t')
		;
	if (this_c == EOF)
		return 0;
	if (this_c == '\\') {
		this_c = getc(fin);
		if (this_c == '\n') {
			bump_lineno();
			return yylex();
		}
	}
	if (this_c == '#') {		/* comment */
		while ((this_c=getc(fin)) != '\n' && this_c != EOF)
			;
		if (this_c == '\n')
			bump_lineno();
		return this_c;
	}
	if (this_c == '.' || isdigit(this_c)) {	/* number */
		fp_t d;
		(void)ungetc(this_c, fin);
		(void)fscanf(fin, "%lf", &d); /* NB: this is really irl_fscanf(), which handles long double case correctly */
		yylval.sym = install("", NUMBER, d);
#if defined(DEBUG_YYLEX)
		(void)fprintf(stderr,"yylex() -> NUMBER [%lg]\n", d);
#endif
		return NUMBER;
	}
	if (IsIdStart(this_c)) {
		Symbol *s;
		(void)ungetc(this_c,fin);
		yylval.sym = s = yygetid();
		return s->type == UNDEF ? VAR : s->type;
	}
	if (this_c == '$') {	/* argument? */
		int n = 0;
		long type;
		this_c = getc(fin);
		if (this_c == '$')	/* $$n means string argument instead of numeric argument, $n */
			type = STRVAR;
		else
		{
			type = VAR;
			(void)ungetc(this_c, fin);
		}
		while (isdigit(this_c=getc(fin)))
			n = 10 * n + this_c - '0';
		(void)ungetc(this_c, fin);
		if (n == 0)
			execerror("strange $...", (const char *)NULL);
		yylval.narg = n;
#if defined(DEBUG_YYLEX)
		(void)fprintf(stderr,"yylex() -> %s [%d]\n", ((type == VAR) ? "ARG" : "STRARG"), n);
#endif
		return (type == VAR) ? ARG : STRARG;
	}
	if (this_c == QUOTATION_MARK) {	/* quoted string */
		const char *sbuf;
		sbuf = yygetstr();
#if 0
		/* Old ugly code that masquerades a string on top of a Symbol */
		yylval.sym = (Symbol *)emalloc(strlen(sbuf)+1);
		strcpy((char *)yylval.sym, sbuf);
#else
		/* New code to generate a real Symbol instead */
#if 0
		yylval.sym = (Symbol *)emalloc(sizeof(Symbol));
		yylval.sym->name = (const char *)NULL;
		yylval.sym->type = STRING;
		yylval.sym->u.str = dupstr(sbuf);
		make_immutable(yylval.sym);
		yylval.sym->next = (Symbol*)NULL;
#else
		yylval.sym = install_const_string("", sbuf);
#endif
#endif
#if defined(DEBUG_YYLEX)
		(void)fprintf(stderr,"yylex() -> STRING [%s]\n", sbuf);
#endif
		return STRING;
	}
	switch (this_c) {
	case '^':	return TO_POWER;
	case ':':	return follow('=', CONSTEQ, ':');
	case '+':	return follow('+', INC, follow('=', ADDEQ, '+'));
	case '-':	return follow('-', DEC, follow('=', SUBEQ, '-'));
	case '*':	return follow('=', MULEQ, follow('*', TO_POWER, '*'));
	case '/':	return follow('=', DIVEQ, '/');
	case '%':	return follow('=', MODEQ, '%');
	case '>':	return follow('=', GE, GT);
	case '<':	{
				if (follow('=', LE, LT) == LE)
					return (LE);
				else if (follow('>', LTGT, LT) == LTGT)
					return (LTGT);
				else
					return (LT);
			}
	case '=':	return follow('=', EQ, '=');
	case '!':	return follow('=', NE, NOT);
	case '|':	return follow('|', OR, '|');
	case '&':	return follow('&', AND, '&');
	case '\n':	bump_lineno(); return '\n';
	default:	return this_c;
	}
}
static YYCONST yytabelem yyexca[] ={
-1, 1,
	0, -1,
	-2, 0,
	};
# define YYNPROD 201
# define YYLAST 2278
static YYCONST yytabelem yyact[]={

   128,   375,    50,    79,   180,   241,   230,     6,    73,    96,
   162,   163,   164,   165,   166,   167,   240,   383,   161,   160,
   242,    78,   125,    50,    50,    50,    76,   244,   243,    44,
    99,    77,    46,    10,    45,   374,   100,   101,   327,   240,
   306,   363,   386,    78,   359,    50,   228,   197,    76,    74,
   357,    75,    50,    77,   358,   229,     2,   359,   319,   103,
    66,    67,    68,    69,    70,    93,    97,   356,   336,   335,
   334,   333,   332,   331,   330,   328,   201,   326,   310,   289,
   318,   317,    11,   239,   227,   199,   179,   178,   177,   176,
   175,   174,    50,   173,   172,   171,   119,   170,   154,   153,
   152,   151,   150,   149,    50,    50,   148,   147,   146,   145,
   144,   143,    28,    50,    50,   142,   139,   138,   137,   130,
   129,   200,    90,    98,     5,   127,    50,     4,   392,   389,
   381,   360,    73,    96,     3,    96,    73,   102,     1,    27,
    26,    50,    25,   140,   136,   232,   233,   168,   236,   133,
    50,   141,    50,    50,    50,    50,    96,    73,   112,     0,
     0,     0,     0,   299,   301,   302,   303,   304,   305,     0,
    96,     0,     0,   287,   286,    50,     0,    50,    50,    50,
    50,    50,    50,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,   235,    50,    50,
     0,     0,     0,    73,    96,    73,    96,     0,     0,     0,
     0,     0,    73,    96,    73,    96,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,    96,    50,    50,
    50,     0,     0,     0,     0,     0,     0,     0,    73,    96,
    50,     0,     0,     0,     0,     0,    29,    55,    22,    14,
   181,    52,    53,    54,     0,    30,    21,    20,     0,     0,
    31,    32,    36,    37,    38,    39,    18,    19,    33,    34,
    35,    56,    57,    40,    41,    58,    23,    24,    51,    59,
    60,    61,    62,    15,    63,    43,    42,    64,    65,    16,
    17,     0,    96,   198,    96,    96,    96,    96,   329,     0,
   105,   106,   107,   108,   109,   104,    79,    50,    47,    48,
    49,     0,    96,     0,    96,    96,    96,    96,   110,   111,
     0,    73,    96,    73,    96,    73,    96,     0,    79,    50,
     0,   135,     0,     7,   361,     0,   362,   120,   121,   122,
   123,   124,    50,     0,     0,     0,    73,    96,     0,     0,
     0,     0,   126,    50,   131,     0,   131,    50,    50,     0,
    50,    50,     0,     0,   369,   370,    96,    96,   373,     0,
     0,     0,    73,    96,     0,     0,   156,   157,   158,   159,
     0,   382,     0,   385,    50,     0,     0,     0,     0,   390,
    50,   387,     0,     0,   393,     0,     0,   391,     0,   114,
   115,   116,   117,   118,   113,     0,   182,   183,   184,   185,
   186,   187,   188,   189,   190,   191,   192,   193,   194,   195,
   196,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,   202,   204,   206,   207,   208,
   209,   210,     0,     0,   211,   213,   215,   216,   217,   218,
   219,   220,   221,   222,   223,   224,   225,     0,     0,     0,
     0,     0,     0,     0,     0,   238,     0,     8,     0,   231,
   231,   231,   237,     0,     0,     0,    10,     0,   288,     0,
   290,     0,   292,     0,     0,     0,     0,     0,   132,   134,
   132,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,   307,   308,   309,     0,    44,   312,     0,    46,
   155,    45,     0,     0,     0,     0,     0,   169,    78,     0,
     0,     0,   388,    76,    74,    11,    75,     0,    77,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,   155,     0,   320,
   322,   324,     0,     0,     0,     0,     0,    78,     0,   203,
   205,   320,    76,    74,   380,    75,     0,    77,   212,   214,
     0,     0,     0,     0,     0,     0,     0,     0,     0,    28,
     0,   226,     0,     0,     0,     0,     0,     0,     0,     0,
     0,    10,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,   291,     0,   293,   294,   295,
   296,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,    44,     0,     0,    46,     0,    45,     0,   345,     0,
   311,     0,   313,   314,   315,   316,   203,   205,     0,     0,
    11,     0,     0,     0,     0,     0,     0,     0,     0,   231,
   237,     0,     0,   212,   214,     0,     0,     0,     0,     0,
     0,     0,   364,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,   367,     0,   368,   237,   237,
     0,   371,   237,   321,   323,   325,    44,     0,     0,    46,
     0,    45,     0,     0,     0,   321,   231,     0,     0,     0,
     0,     0,   384,     0,    28,   237,   234,     0,     0,     0,
     0,   237,     9,    29,    55,    22,    14,     0,    52,    53,
    54,     0,    30,    21,    20,    12,    13,    31,    32,    36,
    37,    38,    39,    18,    19,    33,    34,    35,    56,    57,
    40,    41,    58,    23,    24,    51,    59,    60,    61,    62,
    15,    63,    43,    42,    64,    65,    16,    17,    44,     0,
     0,    46,   346,    45,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,    10,    47,    48,    49,     0,    88,
    87,    80,    81,    82,    84,    85,    86,    83,     0,     0,
     0,     0,     0,    79,     0,     0,     0,   365,     0,     0,
     0,    78,     0,     0,    91,     0,    76,    74,   366,    75,
     0,    77,     0,     0,     0,   372,    78,     0,     0,     0,
   379,    76,    74,    11,    75,     0,    77,     0,    88,    87,
    80,    81,    82,    84,    85,    86,    83,     0,    29,    55,
    22,    14,    79,    52,    53,    54,     0,    30,    21,    20,
     0,     0,    31,    32,    36,    37,    38,    39,    18,    19,
    33,    34,    35,    56,    57,    40,    41,    58,    23,    24,
    51,    59,    60,    61,    62,    15,    63,    43,    42,    64,
    65,    16,    17,    78,     0,     0,    91,   298,    76,    74,
     0,    75,     0,    77,     0,     0,     0,     0,     0,     0,
    47,    48,    49,    29,    55,     0,    14,     0,     0,     0,
     0,     0,    30,     0,     0,     0,     0,    31,    32,    36,
    37,    38,    39,     0,     0,    33,    34,    35,    56,    57,
    40,    41,    58,     0,     0,    51,    59,    60,    61,    62,
    15,    63,    43,    42,    64,    65,    16,    17,     0,    78,
     0,     0,    91,     0,    76,    74,     0,    75,     0,    77,
     0,     0,     0,     0,     0,    47,    48,    49,     0,     0,
     0,     0,     0,     0,     0,    29,    55,     0,    14,     0,
     0,     0,     0,     0,    30,     0,     0,     0,     0,    31,
    32,    36,    37,    38,    39,     0,     0,    33,    34,    35,
    56,    57,    40,    41,    58,     0,     0,     0,    59,    60,
    61,    62,    15,    63,    43,    42,    64,    65,    16,    17,
     0,    71,    55,    78,    72,     0,     0,   376,    76,    74,
     0,    75,     0,    77,     0,    92,     0,    47,    48,    49,
     0,     0,     0,     0,     0,     0,    56,    57,     0,     0,
    58,     0,     0,     0,    59,    60,    61,    62,    89,    63,
     0,    78,    64,    65,     0,    17,    76,    74,   340,    75,
     0,    77,    88,    87,    80,    81,    82,    84,    85,    86,
    83,     0,     0,     0,     0,     0,    79,    88,    87,    80,
    81,    82,    84,    85,    86,    83,     0,     0,     0,     0,
     0,    79,     0,    71,    55,    78,    72,     0,     0,   351,
    76,    74,     0,    75,     0,    77,     0,    92,     0,     0,
     0,     0,     0,     0,     0,    78,     0,     0,    56,    57,
    76,    74,    58,    75,     0,    77,    59,    60,    61,    62,
    89,    63,     0,    78,    64,    65,     0,    17,    76,    74,
     0,    75,     0,    77,    88,    87,    80,    81,    82,    84,
    85,    86,    83,     0,     0,     0,     0,     0,    79,    71,
    55,    78,    72,     0,     0,   349,    76,    74,     0,    75,
     0,    77,     0,    92,     0,     0,     0,     0,     0,     0,
     0,    78,     0,     0,    56,    57,    76,    74,    58,    75,
     0,    77,    59,    60,    61,    62,    89,    63,     0,     0,
    64,    65,     0,    17,     0,     0,     0,     0,     0,     0,
    88,    87,    80,    81,    82,    84,    85,    86,    83,     0,
     0,     0,     0,     0,    79,   252,   285,   284,   250,   251,
   248,   249,   277,   278,   247,   246,   245,   253,   255,   273,
   274,   275,   276,   256,   257,   258,   259,   254,   260,   261,
   262,   263,   269,   265,   266,   267,   279,   280,   281,   282,
   283,   268,   264,   271,   272,   270,    78,    91,   378,     0,
   348,    76,    74,     0,    75,     0,    77,     0,     0,     0,
     0,     0,     0,     0,    88,    87,    80,    81,    82,    84,
    85,    86,    83,     0,     0,    78,     0,     0,    79,   347,
    76,    74,     0,    75,     0,    77,     0,     0,     0,     0,
    78,     0,     0,     0,   338,    76,    74,     0,    75,     0,
    77,     0,    88,    87,    80,    81,    82,    84,    85,    86,
    83,     0,     0,    78,     0,     0,    79,   337,    76,    74,
     0,    75,     0,    77,     0,     0,     0,     0,     0,     0,
     0,     0,    10,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,    88,    87,    80,    81,
    82,    84,    85,    86,    83,     0,     0,     0,     0,     0,
    79,     0,    91,     0,     0,     0,    88,    87,    80,    81,
    82,    84,    85,    86,    83,     0,     0,     0,     0,     0,
    79,    11,     0,     0,     0,    87,    80,    81,    82,    84,
    85,    86,    83,     0,     0,     0,     0,     0,    79,     0,
     0,     0,     0,     0,     0,     0,     0,    91,   377,     0,
     0,     0,    88,    87,    80,    81,    82,    84,    85,    86,
    83,     0,     0,     0,     0,     0,    79,     0,     0,     0,
     0,     0,     0,     0,    80,    81,    82,    84,    85,    86,
    83,     0,     0,     0,     0,     0,    79,     0,     0,     0,
     0,     0,    91,     0,     0,     0,   355,     0,     0,     0,
     0,     0,     0,     0,    94,    55,     0,    95,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,    92,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,    56,
    57,     0,     0,    58,     0,     0,     0,    59,    60,    61,
    62,    89,    63,    91,   354,    64,    65,     0,    17,     0,
     0,     0,     0,     0,     0,     0,     0,    88,    87,    80,
    81,    82,    84,    85,    86,    83,     0,     0,     0,     0,
     0,    79,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,    88,    87,    80,    81,
    82,    84,    85,    86,    83,    91,     0,     0,     0,   353,
    79,    88,    87,    80,    81,    82,    84,    85,    86,    83,
     0,     0,     0,     0,     0,    79,     0,     0,     0,    94,
    55,     0,    95,     0,    88,    87,    80,    81,    82,    84,
    85,    86,    83,    92,     0,     0,     0,     0,    79,     0,
     0,     0,    91,     0,    56,    57,   352,     0,    58,     0,
     0,     0,    59,    60,    61,    62,    89,    63,     0,     0,
    64,    65,     0,    17,    94,    55,     0,    95,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,    92,     0,
     0,     0,     0,     0,     0,     0,     0,    91,   350,    56,
    57,     0,     0,    58,     0,     0,     0,    59,    60,    61,
    62,    89,    63,     0,     0,    64,    65,     0,    17,    94,
    55,     0,    95,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,    92,     0,     0,     0,     0,     0,     0,
     0,     0,    91,   344,    56,    57,     0,     0,    58,     0,
     0,     0,    59,    60,    61,    62,    89,    63,     0,     0,
    64,    65,     0,    17,     0,     0,     0,     0,     0,     0,
    94,    55,     0,    95,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,    92,     0,     0,    91,   343,     0,
     0,     0,     0,     0,     0,    56,    57,     0,     0,    58,
     0,     0,     0,    59,    60,    61,    62,    89,    63,     0,
     0,    64,    65,     0,    17,     0,     0,     0,     0,     0,
     0,     0,    94,    55,     0,    95,     0,     0,     0,     0,
     0,     0,    91,   342,     0,     0,    92,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,    56,    57,     0,
     0,    58,     0,     0,     0,    59,    60,    61,    62,    89,
    63,     0,     0,    64,    65,     0,    17,     0,     0,    94,
    55,     0,    95,     0,     0,     0,     0,    91,     0,     0,
     0,   341,     0,    92,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,    56,    57,     0,     0,    58,     0,
     0,     0,    59,    60,    61,    62,    89,    63,     0,     0,
    64,    65,     0,    17,    94,    55,     0,    95,     0,     0,
     0,     0,    91,   339,     0,     0,     0,     0,    92,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,    56,
    57,     0,     0,    58,     0,     0,     0,    59,    60,    61,
    62,    89,    63,     0,     0,    64,    65,     0,    17,    94,
    55,     0,    95,     0,     0,     0,     0,    91,   297,     0,
     0,     0,     0,    92,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,    56,    57,     0,     0,    58,     0,
     0,     0,    59,    60,    61,    62,    89,    63,     0,     0,
    64,    65,     0,    17,    94,    55,     0,    95,     0,     0,
     0,     0,    91,     0,     0,     0,     0,     0,    92,     0,
     0,     0,     0,     0,     0,     0,    91,     0,     0,    56,
    57,     0,     0,    58,     0,     0,     0,    59,    60,    61,
    62,    89,    63,     0,     0,    64,    65,     0,    17,    94,
    55,     0,    95,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,    92,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,    56,    57,     0,     0,    58,     0,
     0,     0,    59,    60,    61,    62,    89,    63,     0,     0,
    64,    65,     0,    17,    94,    55,     0,    95,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,    92,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,    56,
    57,     0,     0,    58,     0,     0,     0,    59,    60,    61,
    62,    89,    63,     0,     0,    64,    65,     0,    17,    94,
    55,     0,    95,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,    92,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,    56,    57,     0,     0,    58,     0,
     0,     0,    59,    60,    61,    62,    89,    63,     0,     0,
    64,    65,     0,    17,    94,    55,     0,    95,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,    92,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,    56,
    57,     0,     0,    58,     0,     0,     0,    59,    60,    61,
    62,    89,    63,     0,     0,    64,    65,     0,    17,    94,
    55,     0,    95,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,    92,    55,     0,   300,     0,     0,     0,
     0,     0,     0,     0,    56,    57,     0,    92,    58,     0,
     0,     0,    59,    60,    61,    62,    89,    63,    56,    57,
    64,    65,    58,    17,     0,     0,    59,    60,    61,    62,
    89,    63,     0,     0,    64,    65,     0,    17 };
static YYCONST yytabelem yypact[]={

-10000000,   466,-10000000,    23,    23,    23,    23,   774,  1362,    23,
-10000000,-10000000,  -230,  -230,    -2,    97,    35,   -39,-10000000,-10000000,
   728,-10000000,   656,   656,   656,    78,    77,    76,-10000000,-10000000,
-10000000,    75,    71,    70,    69,    68,    67,    66,    63,    62,
    61,    60,    59,    58,   656,   728,   728,   728,  -241,  -242,
  -300,   656,-10000000,-10000000,-10000000,-10000000,    57,    55,    54,    53,
    51,    50,    49,    48,    47,    46,-10000000,-10000000,-10000000,-10000000,
-10000000,-10000000,   -57,-10000000,   728,   728,   728,   728,   728,   728,
   728,   728,   728,   728,   728,   728,   728,   728,   728,   -14,
-10000000,   656,    45,-10000000,-10000000,   -57,-10000000,-10000000,-10000000,-10000000,
-10000000,-10000000,-10000000,   656,   656,   728,   728,   728,   728,   728,
-10000000,-10000000,   656,   656,   728,   728,   728,   728,   728,   728,
   728,   728,   728,   728,   728,   656,  1098,-10000000,  -300,    44,
     2,   922,  1962,    11,  1962,   922,     2,   728,   728,   728,
   591,    43,  -255,  -238,   986,   986,   728,    38,   728,   656,
   728,   656,   656,   656,   656,  1917,   856,  -319,  -319,  -319,
-10000000,-10000000,  1976,  1976,  1976,  1976,  1976,  1976,    -4,  1962,
   728,   728,   728,    37,   656,   728,   656,   656,   656,   656,
   656,   656,   -16,   -16,  -319,  -319,  -319,  -319,     6,     6,
     6,     6,     6,     6,     6,  1164,  1116,   656,   656,  -278,
    41,    40,   922,  1962,   922,  1962,  1098,  1098,  1098,  1098,
  1098,   922,  1962,   922,  1962,  1098,  1098,  1098,  1098,  1098,
  1098,  1098,  1098,  1098,  1098,  1098,  1962,   656,   656,   656,
    36,  1098,   -21,    34,-10000000,-10000000,-10000000,   922,  1962,   656,
    33,    32,    31,    30,    29,-10000000,-10000000,-10000000,-10000000,-10000000,
-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,
-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,
-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,
-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,    28,    27,  1316,-10000000,
  1293,  1872,  1034,  1827,  1782,  1737,  1692,-10000000,-10000000,-10000000,
   -57,-10000000,-10000000,-10000000,-10000000,-10000000,   656,  1278,  1249,  1144,
-10000000,  1647,  1078,  1602,  1555,  1503,  1452,    26,     9,    13,
   922,  1962,   922,  1962,   922,  1962,-10000000,   728,   -11,     0,
-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,
   728,   656,-10000000,-10000000,-10000000,   922,  1962,-10000000,-10000000,-10000000,
-10000000,-10000000,   656,   728,-10000000,   728,   -11,   -11,-10000000,   656,
   -11,   -24,-10000000,-10000000,   996,  1407,  1247,   789,   530,-10000000,
-10000000,   922,  1962,-10000000,   728,  -248,-10000000,-10000000,-10000000,-10000000,
   728,-10000000,     1,   -11,   481,-10000000,-10000000,-10000000,-10000000,   -11,
-10000000,-10000000,-10000000,-10000000 };
static YYCONST yytabelem yypgo[]={

     0,   331,     7,   125,   119,   149,   147,   143,     6,   142,
   140,   139,   120,     1,   122,   465,     0,    28,    27,   123,
    58,   138,    56,   134,   131,   130,   129,   128,   121,    76 };
static YYCONST yytabelem yyr1[]={

     0,    21,    21,    21,    21,    21,    21,    21,    21,    21,
    22,    22,     3,     3,     3,     3,     3,     3,     3,     3,
     3,     3,     3,     3,     3,     3,     3,     3,     3,     3,
     3,     3,    14,    14,    14,    14,    14,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,    24,    25,     2,
    26,    27,     2,     2,     2,     2,     8,     9,    10,    11,
    12,    13,    17,    17,    17,    17,    17,    17,    17,    17,
    17,    17,    17,    17,    17,    17,    17,    17,    17,    17,
    17,    17,    17,    17,    17,    17,    17,    17,    17,    17,
    18,    18,    18,    18,    18,    18,    18,    18,    18,    18,
    18,    18,    18,     7,     7,     7,    16,    16,    16,    16,
    16,    16,    16,    16,    16,    16,    16,    16,    16,    16,
    16,    16,    15,    15,    15,    15,    15,    15,    15,    15,
     1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
     1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
     1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
     1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
     1,     1,     1,     1,     1,     1,     1,     1,     1,     4,
     4,     4,     4,     5,     5,     5,     6,     6,     6,    28,
    23,    29,    23,    19,    19,    19,    20,    20,    20,    20,
    20 };
static YYCONST yytabelem yyr2[]={

     0,     0,     4,     6,     7,     7,     7,     7,     7,     7,
     2,     2,     7,     7,     7,     7,     7,     7,     7,     7,
     7,     7,     7,     7,     7,     7,     7,     7,     7,     7,
     7,     7,     7,     7,     7,     7,     7,     3,     3,     3,
     3,     3,     5,    11,     5,     5,     5,     1,     1,    17,
     1,     1,    25,    13,    19,     7,     3,     3,     3,     3,
     1,     1,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     1,     4,     4,     3,     3,     3,     2,
     7,     9,     9,     9,     9,     7,     9,     9,    13,    13,
     9,    17,     2,     5,     5,     5,     5,     5,     5,     5,
     3,     3,     3,     2,    11,     9,     9,     9,     9,     9,
     9,     9,     7,     9,     9,    13,    13,     9,     9,     9,
     7,     7,     7,     7,     7,     7,     7,     5,     5,     7,
     7,     7,     7,     7,     7,     7,     7,     7,     5,     5,
     5,     5,     5,     7,     7,     7,     7,     7,     7,     3,
     3,     7,     7,     3,     7,     7,     3,     7,     7,     1,
    13,     1,    13,     2,     2,     2,     1,     3,     3,     7,
     7 };
static YYCONST yytabelem yychk[]={

-10000000,   -21,   -22,   -23,    -3,   -14,    -2,    -1,   -15,   256,
    10,    59,   269,   270,   260,   294,   300,   301,   277,   278,
   268,   267,   259,   287,   288,    -9,   -10,   -11,   123,   257,
   266,   271,   272,   279,   280,   281,   273,   274,   275,   276,
   284,   285,   297,   296,    40,    45,    43,   319,   320,   321,
   -16,   289,   262,   263,   264,   258,   282,   283,   286,   290,
   291,   292,   293,   295,   298,   299,   -22,   -22,   -22,   -22,
   -22,   257,   260,   -16,    43,    45,    42,    47,    37,   322,
   310,   311,   312,   316,   313,   314,   315,   309,   308,   294,
   -14,    40,   271,   -22,   257,   260,   -16,   -22,   -19,   260,
   266,   267,   -19,    61,   307,   302,   303,   304,   305,   306,
   320,   321,    61,   307,   302,   303,   304,   305,   306,    61,
   302,   303,   304,   305,   306,    61,    -1,    -3,   -16,   -12,
    -4,    -1,   -15,    -5,   -15,    -1,    -4,    40,    40,    40,
    -7,   -12,    40,    40,    40,    40,    40,    40,    40,    40,
    40,    40,    40,    40,    40,   -15,    -1,    -1,    -1,    -1,
   260,   260,   310,   311,   312,   313,   314,   315,    -6,   -15,
    40,    40,    40,    40,    40,    40,    40,    40,    40,    40,
    61,   307,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
    -1,    -1,    -1,    -1,    -1,    -1,    -1,    61,   307,    40,
   -28,   -29,    -1,   -15,    -1,   -15,    -1,    -1,    -1,    -1,
    -1,    -1,   -15,    -1,   -15,    -1,    -1,    -1,    -1,    -1,
    -1,    -1,    -1,    -1,    -1,    -1,   -15,    40,    44,    44,
    -8,    -1,    -8,    -8,   125,   -22,    -2,    -1,   -15,    40,
   294,   260,   258,   -17,   -18,   270,   269,   268,   264,   265,
   262,   263,   259,   271,   281,   272,   277,   278,   279,   280,
   282,   283,   284,   285,   296,   287,   288,   289,   295,   286,
   299,   297,   298,   273,   274,   275,   276,   266,   267,   290,
   291,   292,   293,   294,   261,   260,   -17,   -18,    -1,    41,
    -1,   -15,    -1,   -15,   -15,   -15,   -15,    41,    41,   -16,
   260,   -16,   -16,   -16,   -16,   -16,    44,    -1,    -1,    -1,
    41,   -15,    -1,   -15,   -15,   -15,   -15,    40,    40,   -20,
    -1,   -15,    -1,   -15,    -1,   -15,    41,    59,    41,   -20,
    41,    41,    41,    41,    41,    41,    41,    41,    41,    41,
    44,    44,    41,    41,    41,    -1,   -15,    41,    41,    41,
    41,    41,    44,    44,    41,    44,    41,    41,    41,    44,
   -24,    -8,    -2,    41,    -1,   -15,   -15,    -1,    -1,    -2,
    -2,    -1,   -15,    -2,    59,   -13,    41,    41,    41,    41,
    44,   -25,    -8,   265,    -1,   -13,    41,    -2,    41,   -26,
   -13,    -2,   -27,   -13 };
static YYCONST yytabelem yydef[]={

     1,    -2,     2,     0,   133,   109,     0,     0,     0,     0,
    10,    11,     0,     0,   131,   107,   132,   108,    39,    40,
    41,    60,     0,     0,     0,     0,     0,     0,   103,   130,
    60,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   122,     0,    57,    58,    59,   106,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     3,     4,     5,     6,
     7,   123,   124,   125,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,   107,
   109,     0,     0,     8,   126,   127,   128,     9,   189,   193,
   194,   195,   191,     0,     0,     0,     0,     0,     0,     0,
   171,   172,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,    42,   133,     0,     0,
    44,   179,   180,    45,   183,     0,    46,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,   157,   158,   168,
   169,   170,     0,     0,     0,     0,     0,     0,   129,   186,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,   151,   152,   153,   154,   155,   156,   159,   160,
   161,   162,   163,   164,   165,   166,   167,     0,     0,     0,
     0,     0,    12,    35,    13,    36,    14,    15,    16,    17,
    18,    19,    32,    20,    33,    21,    22,    23,    24,    25,
    26,    27,    28,    29,    30,    31,    34,   196,     0,     0,
     0,    56,     0,     0,    55,   104,   105,    37,    38,   196,
     0,     0,     0,     0,     0,    62,    63,    64,    65,    66,
    67,    68,    69,    70,    71,    72,    73,    74,    75,    76,
    77,    78,    79,    80,    81,    82,    83,    84,    85,    86,
    87,    88,    89,    90,    91,    92,    93,    94,    95,    96,
    97,    98,    99,   100,   101,   102,     0,     0,     0,   142,
     0,     0,     0,     0,     0,     0,     0,   110,   150,   173,
     0,   174,   175,   176,   177,   178,     0,     0,     0,     0,
   115,     0,     0,     0,     0,     0,     0,     0,     0,     0,
   197,   198,   181,   182,   184,   185,    47,     0,     0,     0,
   114,   135,   136,   137,   138,   139,   140,   141,   143,   144,
     0,     0,   147,   148,   149,   187,   188,   111,   112,   113,
   116,   117,     0,     0,   120,     0,     0,     0,    43,     0,
     0,     0,    61,   134,     0,     0,     0,     0,     0,   190,
   192,   199,   200,    48,     0,    53,   145,   146,   118,   119,
     0,    61,     0,     0,     0,    49,    50,    61,   121,     0,
    54,    51,    61,    52 };
typedef struct
#ifdef __cplusplus
	yytoktype
#endif
{ char *t_name; int t_val; } yytoktype;
#ifndef YYDEBUG
#	define YYDEBUG	0	/* don't allow debugging */
#endif

#if YYDEBUG

yytoktype yytoks[] =
{
	"NUMBER",	257,
	"STRING",	258,
	"PRINT",	259,
	"VAR",	260,
	"UNDEF",	261,
	"WHILE",	262,
	"FOR",	263,
	"IF",	264,
	"ELSE",	265,
	"FUNCTION",	266,
	"PROCEDURE",	267,
	"RETURN",	268,
	"FUNC",	269,
	"PROC",	270,
	"READ",	271,
	"ABORT",	272,
	"BLTIN0",	273,
	"BLTIN1",	274,
	"BLTIN1S",	275,
	"BLTIN2",	276,
	"BREAK",	277,
	"CONTINUE",	278,
	"DEFINED",	279,
	"DELETE",	280,
	"HEX",	281,
	"HEXFP",	282,
	"HEXINT",	283,
	"INDEX",	284,
	"LENGTH",	285,
	"NUMTOSTR",	286,
	"PRINTF",	287,
	"PRINTLN",	288,
	"SPRINTF",	289,
	"STRBLTIN0",	290,
	"STRBLTIN1",	291,
	"STRBLTIN1N",	292,
	"STRBLTIN2",	293,
	"STRVAR",	294,
	"STRFTIME",	295,
	"STRTONUM",	296,
	"STRTOSYMNUM",	297,
	"STRTOSYMSTR",	298,
	"SUBSTR",	299,
	"ARG",	300,
	"STRARG",	301,
	"=",	61,
	"ADDEQ",	302,
	"SUBEQ",	303,
	"MULEQ",	304,
	"DIVEQ",	305,
	"MODEQ",	306,
	"CONSTEQ",	307,
	"OR",	308,
	"AND",	309,
	"GT",	310,
	"GE",	311,
	"LT",	312,
	"LE",	313,
	"EQ",	314,
	"NE",	315,
	"LTGT",	316,
	"+",	43,
	"-",	45,
	"*",	42,
	"/",	47,
	"%",	37,
	"UNARYMINUS",	317,
	"UNARYPLUS",	318,
	"NOT",	319,
	"INC",	320,
	"DEC",	321,
	"TO_POWER",	322,
	"-unknown-",	-1	/* ends search */
};

char * yyreds[] =
{
	"-no such reduction-",
	"list : /* empty */",
	"list : list eos",
	"list : list defn eos",
	"list : list asgn eos",
	"list : list strasgn eos",
	"list : list stmt eos",
	"list : list expr eos",
	"list : list strexpr eos",
	"list : list error eos",
	"eos : '\n'",
	"eos : ';'",
	"asgn : VAR '=' expr",
	"asgn : VAR CONSTEQ expr",
	"asgn : VAR ADDEQ expr",
	"asgn : VAR SUBEQ expr",
	"asgn : VAR MULEQ expr",
	"asgn : VAR DIVEQ expr",
	"asgn : VAR MODEQ expr",
	"asgn : STRVAR '=' expr",
	"asgn : STRVAR CONSTEQ expr",
	"asgn : STRVAR ADDEQ expr",
	"asgn : STRVAR SUBEQ expr",
	"asgn : STRVAR MULEQ expr",
	"asgn : STRVAR DIVEQ expr",
	"asgn : STRVAR MODEQ expr",
	"asgn : ARG '=' expr",
	"asgn : ARG ADDEQ expr",
	"asgn : ARG SUBEQ expr",
	"asgn : ARG MULEQ expr",
	"asgn : ARG DIVEQ expr",
	"asgn : ARG MODEQ expr",
	"strasgn : STRVAR '=' strexpr",
	"strasgn : STRVAR CONSTEQ strexpr",
	"strasgn : STRARG '=' strexpr",
	"strasgn : VAR '=' strexpr",
	"strasgn : VAR CONSTEQ strexpr",
	"stmt : expr",
	"stmt : strexpr",
	"stmt : BREAK",
	"stmt : CONTINUE",
	"stmt : RETURN",
	"stmt : RETURN expr",
	"stmt : PROCEDURE begin '(' arglist ')'",
	"stmt : PRINT prlist",
	"stmt : PRINTF prflist",
	"stmt : PRINTLN prlist",
	"stmt : while '(' cond ')'",
	"stmt : while '(' cond ')' stmt",
	"stmt : while '(' cond ')' stmt end",
	"stmt : for '(' cond ';' cond ';' cond ')'",
	"stmt : for '(' cond ';' cond ';' cond ')' stmt",
	"stmt : for '(' cond ';' cond ';' cond ')' stmt end",
	"stmt : if '(' cond ')' stmt end",
	"stmt : if '(' cond ')' stmt end ELSE stmt end",
	"stmt : '{' stmtlist '}'",
	"cond : expr",
	"while : WHILE",
	"for : FOR",
	"if : IF",
	"begin : /* empty */",
	"end : /* empty */",
	"keyword : PROC",
	"keyword : FUNC",
	"keyword : RETURN",
	"keyword : IF",
	"keyword : ELSE",
	"keyword : WHILE",
	"keyword : FOR",
	"keyword : PRINT",
	"keyword : READ",
	"keyword : HEX",
	"keyword : ABORT",
	"keyword : BREAK",
	"keyword : CONTINUE",
	"keyword : DEFINED",
	"keyword : DELETE",
	"keyword : HEXFP",
	"keyword : HEXINT",
	"keyword : INDEX",
	"keyword : LENGTH",
	"keyword : STRTONUM",
	"keyword : PRINTF",
	"keyword : PRINTLN",
	"keyword : SPRINTF",
	"keyword : STRFTIME",
	"keyword : NUMTOSTR",
	"keyword : SUBSTR",
	"keyword : STRTOSYMNUM",
	"keyword : STRTOSYMSTR",
	"othername : BLTIN0",
	"othername : BLTIN1",
	"othername : BLTIN1S",
	"othername : BLTIN2",
	"othername : FUNCTION",
	"othername : PROCEDURE",
	"othername : STRBLTIN0",
	"othername : STRBLTIN1",
	"othername : STRBLTIN1N",
	"othername : STRBLTIN2",
	"othername : STRVAR",
	"othername : UNDEF",
	"othername : VAR",
	"stmtlist : /* empty */",
	"stmtlist : stmtlist eos",
	"stmtlist : stmtlist stmt",
	"string : STRING",
	"string : STRVAR",
	"string : STRARG",
	"string : strasgn",
	"string : '(' strexpr ')'",
	"string : HEXFP '(' expr ')'",
	"string : HEXINT '(' expr ')'",
	"string : NUMTOSTR '(' expr ')'",
	"string : READ '(' STRVAR ')'",
	"string : STRBLTIN0 '(' ')'",
	"string : STRBLTIN1 '(' strexpr ')'",
	"string : STRBLTIN1N '(' expr ')'",
	"string : STRBLTIN2 '(' strexpr ',' strexpr ')'",
	"string : STRFTIME '(' strexpr ',' expr ')'",
	"string : STRTOSYMSTR '(' strexpr ')'",
	"string : SUBSTR '(' strexpr ',' expr ',' expr ')'",
	"strexpr : string",
	"strexpr : expr NUMBER",
	"strexpr : expr VAR",
	"strexpr : expr string",
	"strexpr : strexpr NUMBER",
	"strexpr : strexpr VAR",
	"strexpr : strexpr string",
	"strexpr : SPRINTF sprflist",
	"expr : NUMBER",
	"expr : VAR",
	"expr : ARG",
	"expr : asgn",
	"expr : FUNCTION begin '(' arglist ')'",
	"expr : READ '(' VAR ')'",
	"expr : ABORT '(' STRING ')'",
	"expr : DEFINED '(' keyword ')'",
	"expr : DEFINED '(' othername ')'",
	"expr : DELETE '(' keyword ')'",
	"expr : DELETE '(' othername ')'",
	"expr : HEX '(' expr ')'",
	"expr : BLTIN0 '(' ')'",
	"expr : BLTIN1 '(' expr ')'",
	"expr : BLTIN1S '(' strexpr ')'",
	"expr : BLTIN2 '(' expr ',' expr ')'",
	"expr : INDEX '(' strexpr ',' strexpr ')'",
	"expr : LENGTH '(' strexpr ')'",
	"expr : STRTOSYMNUM '(' strexpr ')'",
	"expr : STRTONUM '(' strexpr ')'",
	"expr : '(' expr ')'",
	"expr : expr '+' expr",
	"expr : expr '-' expr",
	"expr : expr '*' expr",
	"expr : expr '/' expr",
	"expr : expr '%' expr",
	"expr : expr TO_POWER expr",
	"expr : '-' expr",
	"expr : '+' expr",
	"expr : expr GT expr",
	"expr : expr GE expr",
	"expr : expr LT expr",
	"expr : expr LTGT expr",
	"expr : expr LE expr",
	"expr : expr EQ expr",
	"expr : expr NE expr",
	"expr : expr AND expr",
	"expr : expr OR expr",
	"expr : NOT expr",
	"expr : INC VAR",
	"expr : DEC VAR",
	"expr : VAR INC",
	"expr : VAR DEC",
	"expr : string GT string",
	"expr : string GE string",
	"expr : string LT string",
	"expr : string LE string",
	"expr : string EQ string",
	"expr : string NE string",
	"prlist : expr",
	"prlist : strexpr",
	"prlist : prlist ',' expr",
	"prlist : prlist ',' strexpr",
	"prflist : strexpr",
	"prflist : prflist ',' expr",
	"prflist : prflist ',' strexpr",
	"sprflist : strexpr",
	"sprflist : sprflist ',' expr",
	"sprflist : sprflist ',' strexpr",
	"defn : FUNC procname",
	"defn : FUNC procname '(' ')' stmt",
	"defn : PROC procname",
	"defn : PROC procname '(' ')' stmt",
	"procname : VAR",
	"procname : FUNCTION",
	"procname : PROCEDURE",
	"arglist : /* empty */",
	"arglist : expr",
	"arglist : strexpr",
	"arglist : arglist ',' expr",
	"arglist : arglist ',' strexpr",
};
#endif /* YYDEBUG */
# line	1 "/usr/ccs/bin/yaccpar"
/*
 * Copyright (c) 1993 by Sun Microsystems, Inc.
 */

#pragma ident	"@(#)yaccpar	6.16	99/01/20 SMI"

/*
** Skeleton parser driver for yacc output
*/

/*
** yacc user known macros and defines
*/
#define YYERROR		goto yyerrlab
#define YYACCEPT	return(0)
#define YYABORT		return(1)
#define YYBACKUP( newtoken, newvalue )\
{\
	if ( yychar >= 0 || ( yyr2[ yytmp ] >> 1 ) != 1 )\
	{\
		yyerror( "syntax error - cannot backup" );\
		goto yyerrlab;\
	}\
	yychar = newtoken;\
	yystate = *yyps;\
	yylval = newvalue;\
	goto yynewstate;\
}
#define YYRECOVERING()	(!!yyerrflag)
#define YYNEW(type)	malloc(sizeof(type) * yynewmax)
#define YYCOPY(to, from, type) \
	(type *) memcpy(to, (char *) from, yymaxdepth * sizeof (type))
#define YYENLARGE( from, type) \
	(type *) realloc((char *) from, yynewmax * sizeof(type))
#ifndef YYDEBUG
#	define YYDEBUG	1	/* make debugging available */
#endif

/*
** user known globals
*/
int yydebug;			/* set to 1 to get debugging */

/*
** driver internal defines
*/
#define YYFLAG		(-10000000)

/*
** global variables used by the parser
*/
YYSTYPE *yypv;			/* top of value stack */
int *yyps;			/* top of state stack */

int yystate;			/* current state */
int yytmp;			/* extra var (lasts between blocks) */

int yynerrs;			/* number of errors */
int yyerrflag;			/* error recovery flag */
int yychar;			/* current input token number */



#ifdef YYNMBCHARS
#define YYLEX()		yycvtok(yylex())
/*
** yycvtok - return a token if i is a wchar_t value that exceeds 255.
**	If i<255, i itself is the token.  If i>255 but the neither 
**	of the 30th or 31st bit is on, i is already a token.
*/
#if defined(__STDC__) || defined(__cplusplus)
int yycvtok(int i)
#else
int yycvtok(i) int i;
#endif
{
	int first = 0;
	int last = YYNMBCHARS - 1;
	int mid;
	wchar_t j;

	if(i&0x60000000){/*Must convert to a token. */
		if( yymbchars[last].character < i ){
			return i;/*Giving up*/
		}
		while ((last>=first)&&(first>=0)) {/*Binary search loop*/
			mid = (first+last)/2;
			j = yymbchars[mid].character;
			if( j==i ){/*Found*/ 
				return yymbchars[mid].tvalue;
			}else if( j<i ){
				first = mid + 1;
			}else{
				last = mid -1;
			}
		}
		/*No entry in the table.*/
		return i;/* Giving up.*/
	}else{/* i is already a token. */
		return i;
	}
}
#else/*!YYNMBCHARS*/
#define YYLEX()		yylex()
#endif/*!YYNMBCHARS*/

/*
** yyparse - return 0 if worked, 1 if syntax error not recovered from
*/
#if defined(__STDC__) || defined(__cplusplus)
int yyparse(void)
#else
int yyparse()
#endif
{
	register YYSTYPE *yypvt = 0;	/* top of value stack for $vars */

#if defined(__cplusplus) || defined(lint)
/*
	hacks to please C++ and lint - goto's inside
	switch should never be executed
*/
	static int __yaccpar_lint_hack__ = 0;
	switch (__yaccpar_lint_hack__)
	{
		case 1: goto yyerrlab;
		case 2: goto yynewstate;
	}
#endif

	/*
	** Initialize externals - yyparse may be called more than once
	*/
	yypv = &yyv[-1];
	yyps = &yys[-1];
	yystate = 0;
	yytmp = 0;
	yynerrs = 0;
	yyerrflag = 0;
	yychar = -1;

#if YYMAXDEPTH <= 0
	if (yymaxdepth <= 0)
	{
		if ((yymaxdepth = YYEXPAND(0)) <= 0)
		{
			yyerror("yacc initialization error");
			YYABORT;
		}
	}
#endif

	{
		register YYSTYPE *yy_pv;	/* top of value stack */
		register int *yy_ps;		/* top of state stack */
		register int yy_state;		/* current state */
		register int  yy_n;		/* internal state number info */
	goto yystack;	/* moved from 6 lines above to here to please C++ */

		/*
		** get globals into registers.
		** branch to here only if YYBACKUP was called.
		*/
	yynewstate:
		yy_pv = yypv;
		yy_ps = yyps;
		yy_state = yystate;
		goto yy_newstate;

		/*
		** get globals into registers.
		** either we just started, or we just finished a reduction
		*/
	yystack:
		yy_pv = yypv;
		yy_ps = yyps;
		yy_state = yystate;

		/*
		** top of for (;;) loop while no reductions done
		*/
	yy_stack:
		/*
		** put a state and value onto the stacks
		*/
#if YYDEBUG
		/*
		** if debugging, look up token value in list of value vs.
		** name pairs.  0 and negative (-1) are special values.
		** Note: linear search is used since time is not a real
		** consideration while debugging.
		*/
		if ( yydebug )
		{
			register int yy_i;

			printf( "State %d, token ", yy_state );
			if ( yychar == 0 )
				printf( "end-of-file\n" );
			else if ( yychar < 0 )
				printf( "-none-\n" );
			else
			{
				for ( yy_i = 0; yytoks[yy_i].t_val >= 0;
					yy_i++ )
				{
					if ( yytoks[yy_i].t_val == yychar )
						break;
				}
				printf( "%s\n", yytoks[yy_i].t_name );
			}
		}
#endif /* YYDEBUG */
		if ( ++yy_ps >= &yys[ yymaxdepth ] )	/* room on stack? */
		{
			/*
			** reallocate and recover.  Note that pointers
			** have to be reset, or bad things will happen
			*/
			long yyps_index = (yy_ps - yys);
			long yypv_index = (yy_pv - yyv);
			long yypvt_index = (yypvt - yyv);
			int yynewmax;
#ifdef YYEXPAND
			yynewmax = YYEXPAND(yymaxdepth);
#else
			yynewmax = 2 * yymaxdepth;	/* double table size */
			if (yymaxdepth == YYMAXDEPTH)	/* first time growth */
			{
				char *newyys = (char *)YYNEW(int);
				char *newyyv = (char *)YYNEW(YYSTYPE);
				if (newyys != 0 && newyyv != 0)
				{
					yys = YYCOPY(newyys, yys, int);
					yyv = YYCOPY(newyyv, yyv, YYSTYPE);
				}
				else
					yynewmax = 0;	/* failed */
			}
			else				/* not first time */
			{
				yys = YYENLARGE(yys, int);
				yyv = YYENLARGE(yyv, YYSTYPE);
				if (yys == 0 || yyv == 0)
					yynewmax = 0;	/* failed */
			}
#endif
			if (yynewmax <= yymaxdepth)	/* tables not expanded */
			{
				yyerror( "yacc stack overflow" );
				YYABORT;
			}
			yymaxdepth = yynewmax;

			yy_ps = yys + yyps_index;
			yy_pv = yyv + yypv_index;
			yypvt = yyv + yypvt_index;
		}
		*yy_ps = yy_state;
		*++yy_pv = yyval;

		/*
		** we have a new state - find out what to do
		*/
	yy_newstate:
		if ( ( yy_n = yypact[ yy_state ] ) <= YYFLAG )
			goto yydefault;		/* simple state */
#if YYDEBUG
		/*
		** if debugging, need to mark whether new token grabbed
		*/
		yytmp = yychar < 0;
#endif
		if ( ( yychar < 0 ) && ( ( yychar = YYLEX() ) < 0 ) )
			yychar = 0;		/* reached EOF */
#if YYDEBUG
		if ( yydebug && yytmp )
		{
			register int yy_i;

			printf( "Received token " );
			if ( yychar == 0 )
				printf( "end-of-file\n" );
			else if ( yychar < 0 )
				printf( "-none-\n" );
			else
			{
				for ( yy_i = 0; yytoks[yy_i].t_val >= 0;
					yy_i++ )
				{
					if ( yytoks[yy_i].t_val == yychar )
						break;
				}
				printf( "%s\n", yytoks[yy_i].t_name );
			}
		}
#endif /* YYDEBUG */
		if ( ( ( yy_n += yychar ) < 0 ) || ( yy_n >= YYLAST ) )
			goto yydefault;
		if ( yychk[ yy_n = yyact[ yy_n ] ] == yychar )	/*valid shift*/
		{
			yychar = -1;
			yyval = yylval;
			yy_state = yy_n;
			if ( yyerrflag > 0 )
				yyerrflag--;
			goto yy_stack;
		}

	yydefault:
		if ( ( yy_n = yydef[ yy_state ] ) == -2 )
		{
#if YYDEBUG
			yytmp = yychar < 0;
#endif
			if ( ( yychar < 0 ) && ( ( yychar = YYLEX() ) < 0 ) )
				yychar = 0;		/* reached EOF */
#if YYDEBUG
			if ( yydebug && yytmp )
			{
				register int yy_i;

				printf( "Received token " );
				if ( yychar == 0 )
					printf( "end-of-file\n" );
				else if ( yychar < 0 )
					printf( "-none-\n" );
				else
				{
					for ( yy_i = 0;
						yytoks[yy_i].t_val >= 0;
						yy_i++ )
					{
						if ( yytoks[yy_i].t_val
							== yychar )
						{
							break;
						}
					}
					printf( "%s\n", yytoks[yy_i].t_name );
				}
			}
#endif /* YYDEBUG */
			/*
			** look through exception table
			*/
			{
				register YYCONST int *yyxi = yyexca;

				while ( ( *yyxi != -1 ) ||
					( yyxi[1] != yy_state ) )
				{
					yyxi += 2;
				}
				while ( ( *(yyxi += 2) >= 0 ) &&
					( *yyxi != yychar ) )
					;
				if ( ( yy_n = yyxi[1] ) < 0 )
					YYACCEPT;
			}
		}

		/*
		** check for syntax error
		*/
		if ( yy_n == 0 )	/* have an error */
		{
			/* no worry about speed here! */
			switch ( yyerrflag )
			{
			case 0:		/* new error */
				yyerror( "syntax error" );
				goto skip_init;
			yyerrlab:
				/*
				** get globals into registers.
				** we have a user generated syntax type error
				*/
				yy_pv = yypv;
				yy_ps = yyps;
				yy_state = yystate;
			skip_init:
				yynerrs++;
				/* FALLTHRU */
			case 1:
			case 2:		/* incompletely recovered error */
					/* try again... */
				yyerrflag = 3;
				/*
				** find state where "error" is a legal
				** shift action
				*/
				while ( yy_ps >= yys )
				{
					yy_n = yypact[ *yy_ps ] + YYERRCODE;
					if ( yy_n >= 0 && yy_n < YYLAST &&
						yychk[yyact[yy_n]] == YYERRCODE)					{
						/*
						** simulate shift of "error"
						*/
						yy_state = yyact[ yy_n ];
						goto yy_stack;
					}
					/*
					** current state has no shift on
					** "error", pop stack
					*/
#if YYDEBUG
#	define _POP_ "Error recovery pops state %d, uncovers state %d\n"
					if ( yydebug )
						printf( _POP_, *yy_ps,
							yy_ps[-1] );
#	undef _POP_
#endif
					yy_ps--;
					yy_pv--;
				}
				/*
				** there is no state on stack with "error" as
				** a valid shift.  give up.
				*/
				YYABORT;
			case 3:		/* no shift yet; eat a token */
#if YYDEBUG
				/*
				** if debugging, look up token in list of
				** pairs.  0 and negative shouldn't occur,
				** but since timing doesn't matter when
				** debugging, it doesn't hurt to leave the
				** tests here.
				*/
				if ( yydebug )
				{
					register int yy_i;

					printf( "Error recovery discards " );
					if ( yychar == 0 )
						printf( "token end-of-file\n" );
					else if ( yychar < 0 )
						printf( "token -none-\n" );
					else
					{
						for ( yy_i = 0;
							yytoks[yy_i].t_val >= 0;
							yy_i++ )
						{
							if ( yytoks[yy_i].t_val
								== yychar )
							{
								break;
							}
						}
						printf( "token %s\n",
							yytoks[yy_i].t_name );
					}
				}
#endif /* YYDEBUG */
				if ( yychar == 0 )	/* reached EOF. quit */
					YYABORT;
				yychar = -1;
				goto yy_newstate;
			}
		}/* end if ( yy_n == 0 ) */
		/*
		** reduction by production yy_n
		** put stack tops, etc. so things right after switch
		*/
#if YYDEBUG
		/*
		** if debugging, print the string that is the user's
		** specification of the reduction which is just about
		** to be done.
		*/
		if ( yydebug )
			printf( "Reduce by (%d) \"%s\"\n",
				yy_n, yyreds[ yy_n ] );
#endif
		yytmp = yy_n;			/* value to switch over */
		yypvt = yy_pv;			/* $vars top of value stack */
		/*
		** Look in goto table for next state
		** Sorry about using yy_state here as temporary
		** register variable, but why not, if it works...
		** If yyr2[ yy_n ] doesn't have the low order bit
		** set, then there is no action to be done for
		** this reduction.  So, no saving & unsaving of
		** registers done.  The only difference between the
		** code just after the if and the body of the if is
		** the goto yy_stack in the body.  This way the test
		** can be made before the choice of what to do is needed.
		*/
		{
			/* length of production doubled with extra bit */
			register int yy_len = yyr2[ yy_n ];

			if ( !( yy_len & 01 ) )
			{
				yy_len >>= 1;
				yyval = ( yy_pv -= yy_len )[1];	/* $$ = $1 */
				yy_state = yypgo[ yy_n = yyr1[ yy_n ] ] +
					*( yy_ps -= yy_len ) + 1;
				if ( yy_state >= YYLAST ||
					yychk[ yy_state =
					yyact[ yy_state ] ] != -yy_n )
				{
					yy_state = yyact[ yypgo[ yy_n ] ];
				}
				goto yy_stack;
			}
			yy_len >>= 1;
			yyval = ( yy_pv -= yy_len )[1];	/* $$ = $1 */
			yy_state = yypgo[ yy_n = yyr1[ yy_n ] ] +
				*( yy_ps -= yy_len ) + 1;
			if ( yy_state >= YYLAST ||
				yychk[ yy_state = yyact[ yy_state ] ] != -yy_n )
			{
				yy_state = yyact[ yypgo[ yy_n ] ];
			}
		}
					/* save until reenter driver code */
		yystate = yy_state;
		yyps = yy_ps;
		yypv = yy_pv;
	}
	/*
	** code supplied by user is placed in this switch
	*/
	switch( yytmp )
	{
		
case 4:
# line 201 "hoc.y"
{ CODE2(xpop, STOP); return 1; } break;
case 5:
# line 202 "hoc.y"
{ CODE2(xpop, STOP); return 1; } break;
case 6:
# line 203 "hoc.y"
{ CODE(STOP); return 1; } break;
case 7:
# line 204 "hoc.y"
{ CODE2(printtop, STOP); return 1; } break;
case 8:
# line 205 "hoc.y"
{ CODE2(printtopstring, STOP); return 1; } break;
case 9:
# line 206 "hoc.y"
{ yyerrok; } break;
case 12:
# line 213 "hoc.y"
{ CODE3(varpush,(Inst)yypvt[-2].sym,assign); yyval.inst=yypvt[-0].inst; } break;
case 13:
# line 214 "hoc.y"
{ CODE3(varpush,(Inst)yypvt[-2].sym,const_assign); yyval.inst=yypvt[-0].inst; } break;
case 14:
# line 215 "hoc.y"
{ CODE3(varpush,(Inst)yypvt[-2].sym,addeq); yyval.inst=yypvt[-0].inst; } break;
case 15:
# line 216 "hoc.y"
{ CODE3(varpush,(Inst)yypvt[-2].sym,subeq); yyval.inst=yypvt[-0].inst; } break;
case 16:
# line 217 "hoc.y"
{ CODE3(varpush,(Inst)yypvt[-2].sym,muleq); yyval.inst=yypvt[-0].inst; } break;
case 17:
# line 218 "hoc.y"
{ CODE3(varpush,(Inst)yypvt[-2].sym,diveq); yyval.inst=yypvt[-0].inst; } break;
case 18:
# line 219 "hoc.y"
{ CODE3(varpush,(Inst)yypvt[-2].sym,modeq); yyval.inst=yypvt[-0].inst; } break;
case 19:
# line 220 "hoc.y"
{ CODE3(varpush,(Inst)yypvt[-2].sym,assign); yyval.inst=yypvt[-0].inst; } break;
case 20:
# line 221 "hoc.y"
{ CODE3(varpush,(Inst)yypvt[-2].sym,const_assign); yyval.inst=yypvt[-0].inst; } break;
case 21:
# line 222 "hoc.y"
{ CODE3(varpush,(Inst)yypvt[-2].sym,addeq); yyval.inst=yypvt[-0].inst; } break;
case 22:
# line 223 "hoc.y"
{ CODE3(varpush,(Inst)yypvt[-2].sym,subeq); yyval.inst=yypvt[-0].inst; } break;
case 23:
# line 224 "hoc.y"
{ CODE3(varpush,(Inst)yypvt[-2].sym,muleq); yyval.inst=yypvt[-0].inst; } break;
case 24:
# line 225 "hoc.y"
{ CODE3(varpush,(Inst)yypvt[-2].sym,diveq); yyval.inst=yypvt[-0].inst; } break;
case 25:
# line 226 "hoc.y"
{ CODE3(varpush,(Inst)yypvt[-2].sym,modeq); yyval.inst=yypvt[-0].inst; } break;
case 26:
# line 227 "hoc.y"
{ defnonly("$"); CODE2(argassign,(Inst)yypvt[-2].narg); yyval.inst=yypvt[-0].inst;} break;
case 27:
# line 228 "hoc.y"
{ defnonly("$"); CODE2(argaddeq,(Inst)yypvt[-2].narg); yyval.inst=yypvt[-0].inst;} break;
case 28:
# line 229 "hoc.y"
{ defnonly("$"); CODE2(argsubeq,(Inst)yypvt[-2].narg); yyval.inst=yypvt[-0].inst;} break;
case 29:
# line 230 "hoc.y"
{ defnonly("$"); CODE2(argmuleq,(Inst)yypvt[-2].narg); yyval.inst=yypvt[-0].inst;} break;
case 30:
# line 231 "hoc.y"
{ defnonly("$"); CODE2(argdiveq,(Inst)yypvt[-2].narg); yyval.inst=yypvt[-0].inst;} break;
case 31:
# line 232 "hoc.y"
{ defnonly("$"); CODE2(argmodeq,(Inst)yypvt[-2].narg); yyval.inst=yypvt[-0].inst;} break;
case 32:
# line 235 "hoc.y"
{CODE3(varpush,(Inst)yypvt[-2].sym,str_assign); yyval.inst=yypvt[-0].inst; } break;
case 33:
# line 236 "hoc.y"
{CODE3(varpush,(Inst)yypvt[-2].sym,const_str_assign); yyval.inst=yypvt[-0].inst; } break;
case 34:
# line 237 "hoc.y"
{ defnonly("$"); CODE2(argassign,(Inst)yypvt[-2].narg); yyval.inst=yypvt[-0].inst;} break;
case 35:
# line 238 "hoc.y"
{CODE3(varpush,(Inst)yypvt[-2].sym,str_assign); yyval.inst=yypvt[-0].inst; } break;
case 36:
# line 239 "hoc.y"
{CODE3(varpush,(Inst)yypvt[-2].sym,const_str_assign); yyval.inst=yypvt[-0].inst; } break;
case 37:
# line 242 "hoc.y"
{ CODE(xpop); } break;
case 38:
# line 243 "hoc.y"
{ CODE(xpop); } break;
case 39:
# line 244 "hoc.y"
{ looponly("break"); yyval.inst = code(breakstmt); } break;
case 40:
# line 245 "hoc.y"
{ looponly("continue"); yyval.inst = code(continuestmt); } break;
case 41:
# line 246 "hoc.y"
{ defnonly("return"); yyval.inst = code(procret); } break;
case 42:
# line 247 "hoc.y"
{ defnonly("return"); yyval.inst=yypvt[-0].inst; CODE(funcret); } break;
case 43:
# line 248 "hoc.y"
{ yyval.inst = yypvt[-3].inst; CODE3(call, (Inst)yypvt[-4].sym, (Inst)yypvt[-1].narg); } break;
case 44:
# line 249 "hoc.y"
{ yyval.inst = yypvt[-0].inst; } break;
case 45:
# line 250 "hoc.y"
{ yyval.inst = yypvt[-0].inst; } break;
case 46:
# line 251 "hoc.y"
{ yyval.inst = yypvt[-0].inst; CODE(prnl); } break;
case 47:
# line 252 "hoc.y"
{ inloop++; } break;
case 48:
# line 252 "hoc.y"
{ inloop--; } break;
case 49:
# line 252 "hoc.y"
{
		(yypvt[-7].inst)[1] = (Inst)yypvt[-2].inst;	/* body of loop */
		(yypvt[-7].inst)[2] = (Inst)yypvt[-0].inst; } break;
case 50:
# line 255 "hoc.y"
{ inloop++; } break;
case 51:
# line 255 "hoc.y"
{ inloop--; } break;
case 52:
# line 255 "hoc.y"
{
		(yypvt[-11].inst)[1] = (Inst)yypvt[-7].inst;	/* condition */
		(yypvt[-11].inst)[2] = (Inst)yypvt[-5].inst;	/* post loop */
		(yypvt[-11].inst)[3] = (Inst)yypvt[-2].inst;	/* body of loop */
		(yypvt[-11].inst)[4] = (Inst)yypvt[-0].inst; } break;
case 53:
# line 260 "hoc.y"
{	/* else-less if */
		(yypvt[-5].inst)[1] = (Inst)yypvt[-1].inst;	/* thenpart */
		(yypvt[-5].inst)[3] = (Inst)yypvt[-0].inst; } break;
case 54:
# line 263 "hoc.y"
{	/* if with else */
		(yypvt[-8].inst)[1] = (Inst)yypvt[-4].inst;	/* thenpart */
		(yypvt[-8].inst)[2] = (Inst)yypvt[-1].inst;	/* elsepart */
		(yypvt[-8].inst)[3] = (Inst)yypvt[-0].inst; } break;
case 55:
# line 267 "hoc.y"
{ yyval.inst = yypvt[-1].inst; } break;
case 56:
# line 269 "hoc.y"
{ CODE(STOP); } break;
case 57:
# line 271 "hoc.y"
{ yyval.inst = code3(whilecode,STOP,STOP); } break;
case 58:
# line 273 "hoc.y"
{ yyval.inst = code(forcode); CODE4(STOP,STOP,STOP,STOP); } break;
case 59:
# line 275 "hoc.y"
{ yyval.inst = code(ifcode); CODE3(STOP,STOP,STOP); } break;
case 60:
# line 277 "hoc.y"
{ yyval.inst = progp; } break;
case 61:
# line 279 "hoc.y"
{ CODE(STOP); yyval.inst = progp; } break;
case 103:
# line 329 "hoc.y"
{ yyval.inst = progp; } break;
case 106:
# line 334 "hoc.y"
{ yyval.inst = code2(const_str_push, (Inst)yypvt[-0].sym); } break;
case 107:
# line 335 "hoc.y"
{ yyval.inst = code3(varpush, (Inst)yypvt[-0].sym, streval); } break;
case 108:
# line 336 "hoc.y"
{ defnonly("$$"); yyval.inst = code2(strarg, (Inst)yypvt[-0].narg); } break;
case 110:
# line 338 "hoc.y"
{ yyval.inst = yypvt[-1].inst; } break;
case 111:
# line 339 "hoc.y"
{ CODE(hexfp); } break;
case 112:
# line 340 "hoc.y"
{ CODE(hexint); } break;
case 113:
# line 341 "hoc.y"
{ CODE(numtostr); } break;
case 114:
# line 342 "hoc.y"
{ yyval.inst = code2(varread, (Inst)yypvt[-1].sym); } break;
case 115:
# line 343 "hoc.y"
{ CODE2(strbltin0, (Inst)yypvt[-2].sym->u.ptr0); } break;
case 116:
# line 344 "hoc.y"
{ yyval.inst = yypvt[-1].inst; CODE2(strbltin1, (Inst)yypvt[-3].sym->u.ptr1); } break;
case 117:
# line 345 "hoc.y"
{ yyval.inst = yypvt[-1].inst; CODE2(strbltin1n, (Inst)yypvt[-3].sym->u.ptr1); } break;
case 118:
# line 346 "hoc.y"
{ yyval.inst = yypvt[-3].inst; CODE2(strbltin2, (Inst)yypvt[-5].sym->u.ptr2); } break;
case 119:
# line 347 "hoc.y"
{ CODE(str_strftime); yyval.inst = yypvt[-3].inst; } break;
case 120:
# line 348 "hoc.y"
{ CODE(str_to_symstr); yyval.inst = yypvt[-1].inst; } break;
case 121:
# line 349 "hoc.y"
{ CODE(str_substr); yyval.inst = yypvt[-5].inst; } break;
case 123:
# line 353 "hoc.y"
{ CODE3(constpush, (Inst)yypvt[-0].sym, str_concat_nn); } break;
case 124:
# line 354 "hoc.y"
{ CODE3(constpush, (Inst)yypvt[-0].sym, str_concat_ns); } break;
case 125:
# line 355 "hoc.y"
{ CODE(str_concat_ns); } break;
case 126:
# line 356 "hoc.y"
{ CODE3(constpush, (Inst)yypvt[-0].sym, str_concat_sn); } break;
case 127:
# line 357 "hoc.y"
{ CODE3(constpush, (Inst)yypvt[-0].sym, str_concat_sn); } break;
case 128:
# line 358 "hoc.y"
{ CODE(str_concat_ss); } break;
case 129:
# line 359 "hoc.y"
{ yyval.inst = yypvt[-0].inst; } break;
case 130:
# line 362 "hoc.y"
{ yyval.inst = code2(constpush, (Inst)yypvt[-0].sym); } break;
case 131:
# line 363 "hoc.y"
{ yyval.inst = code3(varpush, (Inst)yypvt[-0].sym, eval); } break;
case 132:
# line 364 "hoc.y"
{ defnonly("$"); yyval.inst = code2(arg, (Inst)yypvt[-0].narg); } break;
case 134:
# line 366 "hoc.y"
{ yyval.inst = yypvt[-3].inst; CODE3(call,(Inst)yypvt[-4].sym,(Inst)yypvt[-1].narg); } break;
case 135:
# line 367 "hoc.y"
{ yyval.inst = code2(varread, (Inst)yypvt[-1].sym); } break;
case 136:
# line 368 "hoc.y"
{ yyval.inst = code2(abort_user, (Inst)yypvt[-1].sym); } break;
case 137:
# line 369 "hoc.y"
{ yyval.inst = code2(Defined, (Inst)yypvt[-1].sym); } break;
case 138:
# line 370 "hoc.y"
{ yyval.inst = code2(Defined, (Inst)yypvt[-1].sym); } break;
case 139:
# line 371 "hoc.y"
{ yyval.inst = code2(Delete, (Inst)yypvt[-1].sym); } break;
case 140:
# line 372 "hoc.y"
{ yyval.inst = code2(Delete, (Inst)yypvt[-1].sym); } break;
case 141:
# line 373 "hoc.y"
{ CODE(hex); } break;
case 142:
# line 374 "hoc.y"
{ CODE2(bltin0, (Inst)yypvt[-2].sym->u.ptr0); } break;
case 143:
# line 375 "hoc.y"
{ yyval.inst = yypvt[-1].inst; CODE2(bltin1, (Inst)yypvt[-3].sym->u.ptr1); } break;
case 144:
# line 376 "hoc.y"
{ yyval.inst = yypvt[-1].inst; CODE2(bltin1s, (Inst)yypvt[-3].sym->u.ptr1); } break;
case 145:
# line 377 "hoc.y"
{ yyval.inst = yypvt[-3].inst; CODE2(bltin2, (Inst)yypvt[-5].sym->u.ptr2); } break;
case 146:
# line 378 "hoc.y"
{ CODE(str_index); yyval.inst = yypvt[-1].inst; } break;
case 147:
# line 379 "hoc.y"
{ CODE(str_length); yyval.inst = yypvt[-1].inst; } break;
case 148:
# line 380 "hoc.y"
{ CODE(str_to_symnum); yyval.inst = yypvt[-1].inst; } break;
case 149:
# line 381 "hoc.y"
{ CODE(str_to_num); yyval.inst = yypvt[-1].inst; } break;
case 150:
# line 382 "hoc.y"
{ yyval.inst = yypvt[-1].inst; } break;
case 151:
# line 383 "hoc.y"
{ CODE(add); } break;
case 152:
# line 384 "hoc.y"
{ CODE(sub); } break;
case 153:
# line 385 "hoc.y"
{ CODE(mul); } break;
case 154:
# line 386 "hoc.y"
{ CODE(divop); } break;
case 155:
# line 387 "hoc.y"
{ CODE(mod); } break;
case 156:
# line 388 "hoc.y"
{ CODE(power); } break;
case 157:
# line 389 "hoc.y"
{ yyval.inst=yypvt[-0].inst; CODE(negate); } break;
case 158:
# line 390 "hoc.y"
{ yyval.inst=yypvt[-0].inst; CODE(noop); } break;
case 159:
# line 391 "hoc.y"
{ CODE(gt); } break;
case 160:
# line 392 "hoc.y"
{ CODE(ge); } break;
case 161:
# line 393 "hoc.y"
{ CODE(lt); } break;
case 162:
# line 394 "hoc.y"
{ CODE(ltgt); } break;
case 163:
# line 395 "hoc.y"
{ CODE(le); } break;
case 164:
# line 396 "hoc.y"
{ CODE(eq); } break;
case 165:
# line 397 "hoc.y"
{ CODE(ne); } break;
case 166:
# line 398 "hoc.y"
{ CODE(And); } break;
case 167:
# line 399 "hoc.y"
{ CODE(Or); } break;
case 168:
# line 400 "hoc.y"
{ yyval.inst = yypvt[-0].inst; CODE(Not); } break;
case 169:
# line 401 "hoc.y"
{ yyval.inst = code2(preinc,(Inst)yypvt[-0].sym); } break;
case 170:
# line 402 "hoc.y"
{ yyval.inst = code2(predec,(Inst)yypvt[-0].sym); } break;
case 171:
# line 403 "hoc.y"
{ yyval.inst = code2(postinc,(Inst)yypvt[-1].sym); } break;
case 172:
# line 404 "hoc.y"
{ yyval.inst = code2(postdec,(Inst)yypvt[-1].sym); } break;
case 173:
# line 405 "hoc.y"
{ CODE(str_gt); } break;
case 174:
# line 406 "hoc.y"
{ CODE(str_ge); } break;
case 175:
# line 407 "hoc.y"
{ CODE(str_lt); } break;
case 176:
# line 408 "hoc.y"
{ CODE(str_le); } break;
case 177:
# line 409 "hoc.y"
{ CODE(str_eq); } break;
case 178:
# line 410 "hoc.y"
{ CODE(str_ne); } break;
case 179:
# line 413 "hoc.y"
{ CODE(prexpr); } break;
case 180:
# line 414 "hoc.y"
{ CODE(prstr); } break;
case 181:
# line 415 "hoc.y"
{ CODE2(prsep,prexpr); } break;
case 182:
# line 416 "hoc.y"
{ CODE2(prsep,prstr); } break;
case 183:
# line 419 "hoc.y"
{ CODE(prfmt); } break;
case 184:
# line 420 "hoc.y"
{ CODE(prexpr); } break;
case 185:
# line 421 "hoc.y"
{ CODE(prstr); } break;
case 186:
# line 424 "hoc.y"
{ CODE(sprfmt); } break;
case 187:
# line 425 "hoc.y"
{ CODE2(sprexpr,sprjoin); } break;
case 188:
# line 426 "hoc.y"
{ CODE2(sprstr,sprjoin); } break;
case 189:
# line 429 "hoc.y"
{ yypvt[-0].sym->type=FUNCTION; indef=1; } break;
case 190:
# line 430 "hoc.y"
{ CODE(procret); define(yypvt[-4].sym); indef=0; } break;
case 191:
# line 431 "hoc.y"
{ yypvt[-0].sym->type=PROCEDURE; indef=1; } break;
case 192:
# line 432 "hoc.y"
{ CODE(procret); define(yypvt[-4].sym); indef=0; } break;
case 196:
# line 440 "hoc.y"
{ yyval.narg = 0; } break;
case 197:
# line 441 "hoc.y"
{ yyval.narg = 1; } break;
case 198:
# line 442 "hoc.y"
{ yyval.narg = 1; } break;
case 199:
# line 443 "hoc.y"
{ yyval.narg = yypvt[-2].narg + 1; } break;
case 200:
# line 444 "hoc.y"
{ yyval.narg = yypvt[-2].narg + 1; } break;
# line	531 "/usr/ccs/bin/yaccpar"
	}
	goto yystack;		/* reset registers in driver code */
}

